from enum import Enum, unique


@unique
class Material(Enum):
    """
    Possible materials of objects. An object can have one or more materials.
    """

    CERAMIC = "CERAMIC"
    """"""

    FABRIC = "FABRIC"
    """"""

    FOOD = "FOOD"
    """"""

    GLASS = "GLASS"
    """"""

    METAL = "METAL"
    """"""

    ORGANIC = "ORGANIC"
    """"""

    PAPER = "PAPER"
    """"""

    PLASTIC = "PLASTIC"
    """"""

    RUBBER = "RUBBER"
    """"""

    SOAP = "SOAP"
    """"""

    SPONGE = "SPONGE"
    """"""

    STONE = "STONE"
    """"""

    UNDEFINED = "UNDEFINED"
    """"""

    WAX = "WAX"
    """"""

    WOOD = "WOOD"
    """"""

    @staticmethod
    def verify_material_enum_string(enum_string):
        """
        Returns whether the given string can be successfully converted into an
        Material enum.

        Parameters
        ----------
        enum_string
            The string to be converted into an Material enum.

        Returns
        -------
        boolean
        """
        try:
            _ = Material[enum_string.upper()]
            return True
        except KeyError:
            return False



import argparse
import os
import statistics
import time

import machine_common_sense as mcs

DEFAULT_STEP_COUNT = 20


def parse_args():
    parser = argparse.ArgumentParser(description='Run MCS')
    parser.add_argument(
        'mcs_unity_build_file',
        help='Path to MCS unity build file')
    parser.add_argument(
        'mcs_scene_dir',
        help='MCS JSON scene configuration directory')
    return parser.parse_args()


def run_scene(controller, file_name):
    scene_data = mcs.load_scene_json_file(file_name)

    scene_file_path = file_name
    scene_file_name = scene_file_path[scene_file_path.rfind('/') + 1:]

    if 'name' not in scene_data.keys():
        scene_data['name'] = scene_file_name[0:scene_file_name.find('.')]

    last_step = DEFAULT_STEP_COUNT
    if (
        'goal' in scene_data.keys(
        ) and 'last_step' in scene_data['goal'].keys()
    ):
        last_step = scene_data['goal']['last_step']

    step_time_list = []

    output = controller.start_scene(scene_data)

    for _ in range(output.step_number + 1, last_step + 1):
        start = time.perf_counter()
        output = controller.step('Pass')
        end = time.perf_counter()
        step_time_list.append(end - start)

    output = controller.end_scene()

    return step_time_list


def main():
    args = parse_args()
    file_list = sorted(
        [
            os.path.join(args.mcs_scene_dir, file_name)
            for file_name in os.listdir(args.mcs_scene_dir)
            if os.path.isfile(os.path.join(args.mcs_scene_dir, file_name)) and
            os.path.splitext(file_name)[1] == '.json'
        ]
    )

    print(
        f'FOUND {len(file_list)} SCENE CONFIGURATION FILES... '
        f'STARTING THE MCS UNITY APP...')
    controller = mcs.create_controller(
        config_file_or_dict={},
        unity_app_file_path=args.mcs_unity_build_file)

    scene_time_list = []
    step_time_list_list = []
    step_time_avg_list = []
    step_time_len_list = []
    step_time_max_list = []
    step_time_min_list = []
    step_time_sum_list = []

    for i in range(len(file_list)):
        print('========================================================='
              '=======================')
        print(f'RUNNING FILE {(i + 1)}: {file_list[i]}')
        start = time.perf_counter()
        step_time_list = run_scene(controller, file_list[i])
        end = time.perf_counter()
        scene_time_list.append(end - start)
        step_time_list_list.append(step_time_list)
        step_time_avg_list.append(statistics.mean(step_time_list))
        step_time_len_list.append(len(step_time_list))
        step_time_max_list.append(max(step_time_list))
        step_time_min_list.append(min(step_time_list))
        step_time_sum_list.append(sum(step_time_list))

    print('===================================================='
          '============================')
    print(f'RAN {len(file_list)} SCENES WITH {sum(step_time_len_list)} '
          f'TOTAL STEPS IN {sum(scene_time_list):0.4f} SECONDS')
    print(
        f'Average single step took {statistics.mean(step_time_avg_list):0.4f} '
        f'seconds')
    print(f'Longest single step took {max(step_time_max_list):0.4f} seconds')
    print(f'Shortest single step took {min(step_time_min_list):0.4f} seconds')
    print(
        f'Average single scene took {statistics.mean(scene_time_list):0.4f} '
        f'seconds')
    print(f'Longest single scene took {max(scene_time_list):0.4f} seconds')
    print(f'Shortest single scene took {min(scene_time_list):0.4f} seconds')


if __name__ == "__main__":
    main()



import argparse

import machine_common_sense as mcs

commands = []


def parse_args():
    parser = argparse.ArgumentParser(description='Run MCS')
    parser.add_argument(
        'mcs_unity_build_file',
        help='Path to MCS unity build file')
    parser.add_argument(
        'mcs_config_json_file',
        help='MCS JSON scene configuration file to load')
    parser.add_argument(
        'mcs_command_list_file',
        help='MCS text file with commands to run')
    return parser.parse_args()


def load_command_file(command_text_file):
    with open(command_text_file, 'r') as command_file:
        commands = [line.strip() for line in command_file]
    return commands


def run_commands(controller, config_data, command_data):
    _ = controller.start_scene(config_data)

    for command in command_data:
        _ = controller.step(command)

    controller.end_scene()


def main():
    args = parse_args()
    config_data = mcs.load_scene_json_file(args.mcs_config_json_file)
    command_data = load_command_file(args.mcs_command_list_file)

    controller = mcs.create_controller(
        unity_app_file_path=args.mcs_unity_build_file,
        config_file_or_dict='./run_scripts_config_with_history.ini'
    )

    config_file_path = args.mcs_config_json_file
    config_file_name = config_file_path[config_file_path.rfind('/') + 1:]

    if 'name' not in config_data.keys():
        config_data['name'] = config_file_name[0:config_file_name.find('.')]

    if controller is not None:
        run_commands(controller, config_data, command_data)


if __name__ == "__main__":
    main()



import argparse
import glob

import machine_common_sense as mcs


def parse_args():
    parser = argparse.ArgumentParser(description='Run MCS')
    parser.add_argument(
        'mcs_unity_build_file',
        help='Path to MCS unity build file')
    parser.add_argument(
        'json_input_folder',
        help='MCS JSON scene input folder')
    parser.add_argument(
        'image_output_folder',
        help='MCS image output folder')
    return parser.parse_args()


def main():
    args = parse_args()

    output_folder = f"{args.image_output_folder}/"
    json_file_list = glob.glob(f"{args.json_input_folder}/*.json")

    controller = mcs.create_controller(
        unity_app_file_path=args.mcs_unity_build_file,
        config_file_or_dict='./config_oracle_debug.ini')

    for json_file_name in json_file_list:
        scene_data = mcs.load_scene_json_file(json_file_name)

        if 'name' not in scene_data.keys():
            scene_data['name'] = json_file_name[json_file_name.rfind(
                '/') + 1:json_file_name.rfind('.')]

        output = controller.start_scene(scene_data)

        print(
            'Saving initialization output (scene image and metadata) of '
            'JSON scene config file ' + scene_data['name'])

        with open(output_folder + scene_data['name'] +
                  '.json', 'w') as output_json_file:
            output_json_file.write(str(output))

        output.image_list[0].save(
            fp=f"{output_folder + scene_data['name']}.png")

    controller.end_scene()


if __name__ == "__main__":
    main()



from .config_manager import ConfigManager
from .controller import Controller
from .controller_logger import (ControllerAi2thorFileGenerator,
                                ControllerDebugFileGenerator, ControllerLogger)
from .controller_media import (DepthImageEventHandler, DepthVideoEventHandler,
                               ImageVideoEventHandler,
                               ObjectMaskImageEventHandler,
                               SceneImageEventHandler,
                               SegmentationVideoEventHandler,
                               TopdownVideoEventHandler,
                               UnityTopdownCameraCombinerEventHandler)
from .history_writer import HistoryEventHandler


def add_subscribers(controller: Controller, config: ConfigManager):
    if not controller:
        return

    if config.is_save_debug_json():
        controller.subscribe(
            ControllerDebugFileGenerator())
        controller.subscribe(ControllerAi2thorFileGenerator())
    if config.is_save_debug_images():
        controller.subscribe(DepthImageEventHandler())
        controller.subscribe(SceneImageEventHandler())
        if (config.is_object_masks_enabled()):
            controller.subscribe(ObjectMaskImageEventHandler())
    if config.is_video_enabled():
        controller.subscribe(ImageVideoEventHandler())
        if config.is_top_down_camera():
            controller.subscribe(UnityTopdownCameraCombinerEventHandler())
            # the two top down video generators use the same filename so we
            # shouldn't enable both at the same time
        elif config.is_top_down_plotter():
            controller.subscribe(TopdownVideoEventHandler())
        if (config.is_depth_maps_enabled()):
            controller.subscribe(DepthVideoEventHandler())
        if (config.is_object_masks_enabled()):
            controller.subscribe(SegmentationVideoEventHandler())
    controller.subscribe(ControllerLogger())
    # TODO once we remove evaulation code, we can better handle when,
    # this handler subscribes
    controller.subscribe(HistoryEventHandler())



import unittest
from unittest.mock import MagicMock, patch

from ai2thor.server import Event

import machine_common_sense as mcs
from machine_common_sense.config_manager import (ConfigManager,
                                                 SceneConfiguration)
from machine_common_sense.controller_events import (AfterStepPayload,
                                                    BeforeStepPayload,
                                                    EndScenePayload,
                                                    StartScenePayload)
from machine_common_sense.goal_metadata import GoalMetadata
from machine_common_sense.history_writer import (HistoryEventHandler,
                                                 HistoryWriter, SceneHistory)
from machine_common_sense.step_metadata import StepMetadata

TEST_FILE_NAME = "test_scene_file.json"


class TestHistoryEventHandler(unittest.TestCase):

    scene_config = SceneConfiguration(name=TEST_FILE_NAME)
    config_mngr = ConfigManager({})

    def setUp(self):
        self.config_mngr._config[
            ConfigManager.CONFIG_DEFAULT_SECTION
        ] = {
            ConfigManager.CONFIG_EVALUATION_NAME: "test-name",
            ConfigManager.CONFIG_HISTORY_ENABLED: True,
            ConfigManager.CONFIG_METADATA_TIER: "level1",
        }

        self.histEvents = HistoryEventHandler()

    def test_on_start_scene(self):
        test_payload = {
            "step_number": 0,
            "config": self.config_mngr,
            "scene_config": self.scene_config,
            "output_folder": None,
            "timestamp": "20210831-202203",
            "wrapped_step": {},
            "step_metadata": Event({'screenWidth': 400, 'screenHeight': 600}),
            "step_output": StepMetadata(),
            "restricted_step_output": StepMetadata(),
            "goal": GoalMetadata()
        }

        self.assertIsNone(
            self.histEvents._HistoryEventHandler__history_writer)

        self.histEvents.on_start_scene(StartScenePayload(**test_payload))

        self.assertIsNotNone(
            self.histEvents._HistoryEventHandler__history_writer)
        writer: HistoryWriter = (
            self.histEvents._HistoryEventHandler__history_writer)
        self.assertIsNotNone(writer.current_steps)
        step = writer.current_steps[0]
        self.assertEqual(step['step'], 0)
        self.assertEqual(step['action'], 'Initialize')
        self.assertEqual(step['args'], {})

        self.assertIsNotNone(step['output'])

        self.assertIsNone(step['params'])
        self.assertIsNone(step['classification'])
        self.assertIsNone(step['confidence'])

    def test_on_start_scene_hist_not_enabled(self):
        self.config_mngr._config[
            ConfigManager.CONFIG_DEFAULT_SECTION
        ] = {
            ConfigManager.CONFIG_EVALUATION_NAME: "test-name",
            ConfigManager.CONFIG_HISTORY_ENABLED: False,
            ConfigManager.CONFIG_METADATA_TIER: "level1",
        }
        test_payload = {
            "step_number": 0,
            "config": self.config_mngr,
            "scene_config": self.scene_config,
            "output_folder": None,
            "timestamp": "20210831-202203",
            "wrapped_step": {},
            "step_metadata": Event({'screenWidth': 600, 'screenHeight': 400}),
            "step_output": StepMetadata(),
            "restricted_step_output": StepMetadata(),
            "goal": GoalMetadata()
        }

        self.assertIsNone(
            self.histEvents._HistoryEventHandler__history_writer)

        self.histEvents.on_start_scene(StartScenePayload(**test_payload))

        self.assertIsNone(
            self.histEvents._HistoryEventHandler__history_writer)

    def test_on_before_step_zero(self):
        self.histEvents._HistoryEventHandler__history_writer = MagicMock()
        test_payload = {
            "step_number": 0,
            "config": self.config_mngr,
            "scene_config": self.scene_config,
            "action": "Initialize",
            "habituation_trial": None,
            "goal": GoalMetadata()
        }

        self.histEvents.on_before_step(BeforeStepPayload(**test_payload))

        hist_writer = self.histEvents._HistoryEventHandler__history_writer

        hist_writer.init_timer.assert_called()
        hist_writer.add_step.assert_not_called()

    def test_on_before_step_one(self):
        hist = SceneHistory(
            step=1,
            action=mcs.Action.MOVE_AHEAD.value,
            args=None,
            params=None,
            output=None,
            delta_time_millis=0)
        self.histEvents._HistoryEventHandler__history_item = hist
        self.histEvents._HistoryEventHandler__history_writer = MagicMock()
        test_payload = {
            "step_number": 1,
            "config": self.config_mngr,
            "scene_config": self.scene_config,
            "action": "Initialize",
            "habituation_trial": None,
            "goal": GoalMetadata()
        }

        self.histEvents.on_before_step(BeforeStepPayload(**test_payload))

        hist_writer = self.histEvents._HistoryEventHandler__history_writer

        hist_writer.init_timer.assert_not_called()
        hist_writer.add_step.assert_called_with(hist)

    def test_on_after_step(self):
        test_payload = {
            "step_number": 1,
            "ai2thor_action": mcs.Action.MOVE_AHEAD.value,
            "action_kwargs": {},
            "step_params": {},
            "config": self.config_mngr,
            "scene_config": self.scene_config,
            "goal": GoalMetadata(),
            "output_folder": None,
            "timestamp": "20210831-202203",
            "wrapped_step": StepMetadata(),
            "step_metadata": Event({'screenHeight': 400, 'screenWidth': 600,
                                    'targetIsVisibleAtStart': True}),
            "step_output": StepMetadata(),
            "restricted_step_output": StepMetadata()
        }

        after_step_payload = AfterStepPayload(**test_payload)
        mock_function = ('machine_common_sense.step_metadata.StepMetadata'
                         '.copy_without_depth_or_images')
        with patch(mock_function) as copy_without_depth_or_images_call:
            self.histEvents.on_after_step(after_step_payload)
            copy_without_depth_or_images_call.assert_called()

        hist = self.histEvents._HistoryEventHandler__history_item
        self.assertEqual(hist.step, 1)
        self.assertEqual(hist.action, mcs.Action.MOVE_AHEAD.value)
        self.assertEqual(hist.delta_time_millis, 0)
        self.assertTrue(hist.target_is_visible_at_start)

    def test_on_end_scene(self):
        self.histEvents._HistoryEventHandler__history_writer = HistoryWriter(
            self.scene_config, {}, "20210831-202203")
        self.histEvents._HistoryEventHandler__history_writer.write_history_file = MagicMock()  # noqa: E501
        hist = SceneHistory(
            step=1,
            action=mcs.Action.MOVE_AHEAD.value,
            args=None,
            params=None,
            output=None,
            delta_time_millis=0)
        self.histEvents._HistoryEventHandler__history_item = hist
        test_payload = {
            'step_number': 1,
            'config': self.config_mngr,
            'scene_config': self.scene_config,
            'rating': 'plausible',
            'score': 0.8,
            'report': {
                1: {
                    "rating": "plausible",
                    "score": 0.75,
                    "violations_xy_list": [{"x": 1, "y": 1}],
                    "internal_state": {"test": "some state"},
                }
            },
        }

        self.histEvents.on_end_scene(EndScenePayload(**test_payload))

        hist_writer = self.histEvents._HistoryEventHandler__history_writer
        self.assertEqual(len(hist_writer.current_steps), 1)
        self.assertEqual(
            hist_writer.current_steps[0]["action"],
            mcs.Action.MOVE_AHEAD.value)
        self.assertEqual(
            hist_writer.current_steps[0]["classification"],
            "plausible")
        self.assertEqual(
            hist_writer.current_steps[0]["confidence"],
            0.75)
        self.assertEqual(
            hist_writer.current_steps[0]["violations_xy_list"],
            [
                {
                    "x": 1,
                    "y": 1
                }
            ])
        self.assertEqual(
            hist_writer.current_steps[0]["internal_state"],
            {
                "test": "some state"
            })
        self.assertEqual(
            hist_writer.current_steps[0]["internal_state"],
            {
                "test": "some state"
            })

        hist_writer.write_history_file.assert_called_with("plausible", .8)

    def test_on_end_scene_history_not_enabled(self):
        self.config_mngr._config[
            ConfigManager.CONFIG_DEFAULT_SECTION
        ] = {
            ConfigManager.CONFIG_EVALUATION_NAME: "test-name",
            ConfigManager.CONFIG_HISTORY_ENABLED: False,
            ConfigManager.CONFIG_METADATA_TIER: "level1",
        }
        self.histEvents._HistoryEventHandler__history_writer = MagicMock()  # noqa: E501

        test_payload = {
            'step_number': 1,
            'config': self.config_mngr,
            'scene_config': self.scene_config,
            'rating': 'plausible',
            'score': 0.8,
            'report': {
                1: {
                    "rating": "plausible",
                    "score": 0.75,
                    "violations_xy_list": [{"x": 1, "y": 1}],
                    "internal_state": {"test": "some state"},
                }
            },
        }

        self.histEvents.on_end_scene(EndScenePayload(**test_payload))

        hist_writer = self.histEvents._HistoryEventHandler__history_writer
        hist_writer.add_step.assert_not_called()
        hist_writer.write_history_file.assert_not_called()


if __name__ == '__main__':
    unittest.main()



import unittest

import machine_common_sense as mcs


class TestMaterial(unittest.TestCase):
    def test_verify_material_enum_string(self):
        self.assertEqual(
            mcs.Material.verify_material_enum_string('Ceramic'), True)
        self.assertEqual(
            mcs.Material.verify_material_enum_string('Plastic'), True)
        self.assertEqual(
            mcs.Material.verify_material_enum_string('Foobar'), False)
        self.assertEqual(mcs.Material.verify_material_enum_string(''), False)


if __name__ == '__main__':
    unittest.main()



import argparse
import glob
import logging
import os.path
import subprocess
import time
from typing import Callable, Dict, List, Tuple

import machine_common_sense as mcs
from machine_common_sense.controller import Controller

logger = logging.getLogger('machine_common_sense')
mcs.LoggingConfig.init_logging(mcs.LoggingConfig.get_dev_logging_config())


SCRIPT_FOLDER = os.path.dirname(os.path.abspath(__file__))
BLACK_IMAGE_PATH = SCRIPT_FOLDER + '/black_image.png'


class AbstractRunnerScript():
    def __init__(
        self,
        name: str,
        action_callback: Callable[
            [Dict, mcs.StepMetadata, 'AbstractRunnerScript'],
            Tuple[str, Dict]
        ],
        init_callback: Callable[[Controller], None] = None
    ):
        self._name = name
        args, filenames = self._read_args()
        if not len(filenames):
            print('No matching files found... Exiting')
            exit()
        self.args = args

        debug = (args.save_videos or args.save_gifs or args.debug)
        config_suffix = 'with_debug' if debug else 'no_debug'
        if args.level1:
            config_suffix = 'level1_debug' if debug else 'level1'
        if args.level2:
            config_suffix = 'level2_debug' if debug else 'level2'
        if args.oracle:
            config_suffix = 'oracle_debug' if debug else 'oracle'

        config_file_path = (
            args.config_file or
            SCRIPT_FOLDER + '/config_' + config_suffix + '.ini'
        )

        print('========================================')
        controller = mcs.create_controller(
            unity_app_file_path=args.mcs_unity_build_file,
            unity_cache_version=args.mcs_unity_version,
            config_file_or_dict=config_file_path
        )

        if init_callback is not None:
            init_callback(controller)

        for filename in filenames:
            print('Starting next scene timer')
            begin_time = time.perf_counter()
            scene_name = self.run_scene(
                controller,
                filename,
                action_callback,
                args.last_step,
                args.prefix,
                args.rename
            )
            end_time = time.perf_counter()
            time_diff = end_time - begin_time
            print(f'{scene_name} scene timer: {time_diff:0.4f} seconds')
            if args.save_videos or args.save_gifs:
                # Copy the black image into the debug folder as the last frame.
                frame_image_list = glob.glob(scene_name + '/frame_image_*')
                frame_count = len(frame_image_list)
                black_frame = (
                    scene_name + '/frame_image_' + str(frame_count) + '.png'
                )
                subprocess.call(['cp', BLACK_IMAGE_PATH, black_frame])
            if args.save_videos:
                subprocess.call([
                    'ffmpeg', '-y', '-r', '20', '-i',
                    scene_name + '/frame_image_%d.png',
                    '-vcodec', 'h264', '-vf', 'format=yuv420p',
                    scene_name + '.mp4'
                ])
            if args.save_gifs:
                subprocess.call([
                    'ffmpeg', '-y', '-r', '20', '-i',
                    scene_name + '/frame_image_%d.png',
                    scene_name + '.gif'
                ])

    def _append_subclass_args_to_parser(
        self,
        parser: argparse.ArgumentParser
    ) -> argparse.ArgumentParser:
        # To override
        return parser

    def _read_args(self) -> Tuple[argparse.Namespace, List[str]]:
        parser = argparse.ArgumentParser(description=('Run ' + self._name))
        parser.add_argument(
            '--config_file',
            type=str,
            default=None,
            help='MCS config file override'
        )
        parser.add_argument(
            '--debug',
            default=False,
            action='store_true',
            help='Save debug data (inputs, outputs, and images) to local files'
        )
        parser.add_argument(
            '--last_step',
            default=None,
            help='Scene last step override'
        )
        parser.add_argument(
            '--mcs_unity_build_file',
            type=str,
            default=None,
            help='Path to MCS unity build file'
        )
        parser.add_argument(
            '--mcs_unity_version',
            type=str,
            default=None,
            help='version of MCS Unity executable.  Default: current'
        )
        parser.add_argument(
            '--prefix',
            default=None,
            help='Append a prefix to each output file'
        )
        parser.add_argument(
            '--rename',
            default=None,
            help='Rename each scene and append the corresponding scene ID'
        )
        parser.add_argument(
            '--save-gifs',
            default=False,
            action='store_true',
            help='Save GIF of each MCS scene'
        )
        parser.add_argument(
            '--save-videos',
            default=False,
            action='store_true',
            help='Save video of each MCS scene'
        )

        # Metadata tiers
        parser.add_argument(
            '--level1',
            default=False,
            action='store_true',
            help='Use level 1 metadata tier and save debug data'
        )
        parser.add_argument(
            '--level2',
            default=False,
            action='store_true',
            help='Use level 2 metadata tier and save debug data'
        )
        parser.add_argument(
            '--oracle',
            default=False,
            action='store_true',
            help='Use oracle metadata tier and save debug data'
        )

        parser = self._append_subclass_args_to_parser(parser)
        return self._read_subclass_args(parser)

    def _read_subclass_args(
        self,
        parser: argparse.ArgumentParser
    ) -> Tuple[argparse.Namespace, List[str]]:
        # To override
        return None, []

    def run_scene(
        self,
        controller: mcs.Controller,
        filename: str,
        action_callback: Callable[
            [Dict, mcs.StepMetadata, 'AbstractRunnerScript'],
            Tuple[str, Dict]
        ],
        last_step: int,
        prefix: str,
        rename: bool
    ):
        scene_data = mcs.load_scene_json_file(filename)

        if last_step:
            scene_data['goal'] = scene_data.get('goal', {})
            scene_data['goal']['last_step'] = int(last_step)

        # Add a name to the scene if needed.
        if 'name' not in scene_data.keys():
            scene_data['name'] = filename[0:filename.find('.')]

        # Remove the folder prefix from the scene name if needed.
        scene_data['name'] = (
            scene_data['name'][(scene_data['name'].rfind('/') + 1):]
        )

        # Use the prefix and/or rename arguments for the new scene name.
        scene_name = ((prefix + '_') if prefix else '') + (
            rename or scene_data.get('name', '')
        )

        if rename and 'sceneInfo' in scene_data.get('goal', {}):
            # Rename the scene using its hypercube cell ID.
            scene_id = scene_data['goal']['sceneInfo']['id'][0]
            scene_name = ((scene_name + '_') if scene_name else '') + scene_id

        # Override the scene name.
        scene_data['name'] = scene_name

        step_metadata = controller.start_scene(scene_data)
        action, params = action_callback(scene_data, step_metadata, self)

        while action is not None:
            step_metadata = controller.step(action, **params)
            if step_metadata is None:
                break
            action, params = action_callback(scene_data, step_metadata, self)

        controller.end_scene()

        return scene_data['name']


class SingleFileRunnerScript(AbstractRunnerScript):
    def _append_subclass_args_to_parser(
        self,
        parser: argparse.ArgumentParser
    ) -> argparse.ArgumentParser:
        parser.add_argument(
            'mcs_scene_filename',
            help='Filename of MCS scene to run'
        )
        return parser

    def _read_subclass_args(
        self,
        parser: argparse.ArgumentParser
    ) -> Tuple[argparse.Namespace, List[str]]:
        args = parser.parse_args()
        return args, [args.mcs_scene_filename]


class MultipleFileRunnerScript(AbstractRunnerScript):
    def __init__(
        self,
        name: str,
        action_callback: Callable[
            [Dict, mcs.StepMetadata, 'AbstractRunnerScript'],
            Tuple[str, Dict]
        ],
        init_callback: Callable[[Controller], None] = None
    ):
        super().__init__(name, action_callback, init_callback)
        if self.args.zip_prefix:
            for file_type in (
                (['mp4'] if self.args.save_videos else []) +
                (['gif'] if self.args.save_gifs else [])
            ):
                subprocess.call(
                    ['zip', self.args.zip_prefix + '_' + file_type + '.zip'] +
                    glob.glob(self.args.mcs_scene_prefix + '*.' + file_type)
                )
            subprocess.call(
                ['zip', self.args.zip_prefix + '_frames.zip'] +
                glob.glob(self.args.mcs_scene_prefix + '*/frame_image_*.png')
            )

    def _append_subclass_args_to_parser(
        self,
        parser: argparse.ArgumentParser
    ) -> argparse.ArgumentParser:
        parser.add_argument(
            'mcs_scene_prefix',
            help='Filename prefix of all MCS scenes to run'
        )
        parser.add_argument(
            '--zip-prefix',
            default=None,
            help='Save ZIPs of frames/videos/GIFs with this filename prefix'
        )
        return parser

    def _read_subclass_args(
        self,
        parser: argparse.ArgumentParser
    ) -> Tuple[argparse.Namespace, List[str]]:
        args = parser.parse_args()
        filenames = glob.glob(args.mcs_scene_prefix + '*_debug.json')
        print(
            f'Found {len(filenames)} files matching '
            f'{args.mcs_scene_prefix + "*_debug.json"}'
        )
        if not len(filenames):
            print('No matching files found... trying non-debug files')
            filenames = glob.glob(args.mcs_scene_prefix + '*.json')
            print(
                f'Found {len(filenames)} files matching '
                f'{args.mcs_scene_prefix + "*.json"}'
            )
        return args, sorted(filenames)



from runner_script import MultipleFileRunnerScript


def action_callback(scene_data, step_metadata, runner_script):
    return 'Pass', {}


def main():
    MultipleFileRunnerScript('Interactive Scenes - Pass', action_callback)


if __name__ == "__main__":
    main()



import unittest

import machine_common_sense as mcs


class TestAction(unittest.TestCase):

    def test_repr(self):
        repr_result1 = (
            "<Action.CLOSE_OBJECT: " +
            mcs.Action.CLOSE_OBJECT.value + ", " +
            mcs.Action.CLOSE_OBJECT.key + ", " +
            mcs.Action.CLOSE_OBJECT.desc +
            ">"
        )
        repr_result2 = (
            "<Action.PUT_OBJECT: " +
            mcs.Action.PUT_OBJECT.value + ", " +
            mcs.Action.PUT_OBJECT.key + ", " +
            mcs.Action.PUT_OBJECT.desc +
            ">"
        )
        self.assertEqual(repr(mcs.Action.CLOSE_OBJECT), repr_result1)
        self.assertEqual(repr(mcs.Action.PUT_OBJECT), repr_result2)

    def test_close_object(self):
        self.assertEqual(mcs.Action.CLOSE_OBJECT.value, "CloseObject")
        self.assertEqual(mcs.Action.CLOSE_OBJECT.key, "1")
        self.assertEqual(
            mcs.Action.CLOSE_OBJECT.desc,
            "Close a nearby object. (objectId=string, amount=float "
            "(default:1), objectImageCoordsX=float, objectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("CloseObject"), mcs.Action.CLOSE_OBJECT)
        self.assertEqual(mcs.Action("1"), mcs.Action.CLOSE_OBJECT)

    def test_drop_object(self):
        self.assertEqual(mcs.Action.DROP_OBJECT.value, "DropObject")
        self.assertEqual(mcs.Action.DROP_OBJECT.key, "2")
        self.assertEqual(
            mcs.Action.DROP_OBJECT.desc,
            "Drop an object you are holding. (objectId=string)"
        )
        self.assertEqual(mcs.Action("DropObject"), mcs.Action.DROP_OBJECT)
        self.assertEqual(mcs.Action("2"), mcs.Action.DROP_OBJECT)

    def test_move_ahead(self):
        self.assertEqual(mcs.Action.MOVE_AHEAD.value, "MoveAhead")
        self.assertEqual(mcs.Action.MOVE_AHEAD.key, "w")
        self.assertEqual(
            mcs.Action.MOVE_AHEAD.desc,
            "Move yourself ahead based on your current view."
        )
        self.assertEqual(mcs.Action("MoveAhead"), mcs.Action.MOVE_AHEAD)
        self.assertEqual(mcs.Action("w"), mcs.Action.MOVE_AHEAD)

    def test_move_back(self):
        self.assertEqual(mcs.Action.MOVE_BACK.value, "MoveBack")
        self.assertEqual(mcs.Action.MOVE_BACK.key, "s")
        self.assertEqual(
            mcs.Action.MOVE_BACK.desc,
            "Move yourself back based on your current view."
        )
        self.assertEqual(mcs.Action("MoveBack"), mcs.Action.MOVE_BACK)
        self.assertEqual(mcs.Action("s"), mcs.Action.MOVE_BACK)

    def test_move_left(self):
        self.assertEqual(mcs.Action.MOVE_LEFT.value, "MoveLeft")
        self.assertEqual(mcs.Action.MOVE_LEFT.key, "a")
        self.assertEqual(
            mcs.Action.MOVE_LEFT.desc,
            "Move yourself to your left based on your current view."
        )
        self.assertEqual(mcs.Action("MoveLeft"), mcs.Action.MOVE_LEFT)
        self.assertEqual(mcs.Action("a"), mcs.Action.MOVE_LEFT)

    def test_move_right(self):
        self.assertEqual(mcs.Action.MOVE_RIGHT.value, "MoveRight")
        self.assertEqual(mcs.Action.MOVE_RIGHT.key, "d")
        self.assertEqual(
            mcs.Action.MOVE_RIGHT.desc,
            "Move yourself to your right based on your current view."
        )
        self.assertEqual(mcs.Action("MoveRight"), mcs.Action.MOVE_RIGHT)
        self.assertEqual(mcs.Action("d"), mcs.Action.MOVE_RIGHT)

    def test_open_object(self):
        self.assertEqual(mcs.Action.OPEN_OBJECT.value, "OpenObject")
        self.assertEqual(mcs.Action.OPEN_OBJECT.key, "3")
        self.assertEqual(
            mcs.Action.OPEN_OBJECT.desc,
            "Open a nearby object. (objectId=string, "
            "amount=float (default:1), objectImageCoordsX=float, "
            "objectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("OpenObject"), mcs.Action.OPEN_OBJECT)
        self.assertEqual(mcs.Action("3"), mcs.Action.OPEN_OBJECT)

    def test_pickup_object(self):
        self.assertEqual(mcs.Action.PICKUP_OBJECT.value, "PickupObject")
        self.assertEqual(mcs.Action.PICKUP_OBJECT.key, "4")
        self.assertEqual(
            mcs.Action.PICKUP_OBJECT.desc,
            "Pickup a nearby object and hold it in your hand. "
            "(objectId=string, objectImageCoordsX=float, "
            "objectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("PickupObject"), mcs.Action.PICKUP_OBJECT)
        self.assertEqual(mcs.Action("4"), mcs.Action.PICKUP_OBJECT)

    def test_pull_object(self):
        self.assertEqual(mcs.Action.PULL_OBJECT.value, "PullObject")
        self.assertEqual(mcs.Action.PULL_OBJECT.key, "5")
        self.assertEqual(
            mcs.Action.PULL_OBJECT.desc,
            "Pull a nearby object. (objectId=string, "
            "force=float (default:0.5), "
            "objectImageCoordsX=float, objectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("PullObject"), mcs.Action.PULL_OBJECT)
        self.assertEqual(mcs.Action("5"), mcs.Action.PULL_OBJECT)

    def test_push_object(self):
        self.assertEqual(mcs.Action.PUSH_OBJECT.value, "PushObject")
        self.assertEqual(mcs.Action.PUSH_OBJECT.key, "6")
        self.assertEqual(
            mcs.Action.PUSH_OBJECT.desc,
            "Push a nearby object. (objectId=string, "
            "force=float (default:0.5), "
            "objectImageCoordsX=float, objectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("PushObject"), mcs.Action.PUSH_OBJECT)
        self.assertEqual(mcs.Action("6"), mcs.Action.PUSH_OBJECT)

    def test_torque_object(self):
        self.assertEqual(mcs.Action.TORQUE_OBJECT.value, "TorqueObject")
        self.assertEqual(mcs.Action.TORQUE_OBJECT.key, "8")
        self.assertEqual(
            mcs.Action.TORQUE_OBJECT.desc,
            "Apply torque to a nearby object. (objectId=string, "
            "force=float(default:0.5), objectImageCoordsX=float, "
            "objectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("TorqueObject"), mcs.Action.TORQUE_OBJECT)
        self.assertEqual(mcs.Action("8"), mcs.Action.TORQUE_OBJECT)

    def test_rotate_object(self):
        self.assertEqual(mcs.Action.ROTATE_OBJECT.value, "RotateObject")
        self.assertEqual(mcs.Action.ROTATE_OBJECT.key, "9")
        self.assertEqual(
            mcs.Action.ROTATE_OBJECT.desc,
            "Apply a rotation of 5 degrees to a nearby object. "
            "(objectId=string, "
            "clockwise=bool(default:True), objectImageCoordsX=float, "
            "objectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("RotateObject"), mcs.Action.ROTATE_OBJECT)
        self.assertEqual(mcs.Action("9"), mcs.Action.ROTATE_OBJECT)

    def test_move_object(self):
        self.assertEqual(mcs.Action.MOVE_OBJECT.value, "MoveObject")
        self.assertEqual(mcs.Action.MOVE_OBJECT.key, "m")
        self.assertEqual(
            mcs.Action.MOVE_OBJECT.desc,
            "Apply a movement of 0.1 meters to a nearby object. "
            "(objectId=string, "
            "lateral=int(default:0), "
            "straight=int(default:1), "
            "objectImageCoordsX=float, "
            "objectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("MoveObject"), mcs.Action.MOVE_OBJECT)
        self.assertEqual(mcs.Action("m"), mcs.Action.MOVE_OBJECT)

    def test_interact_with_agent(self):
        self.assertEqual(
            mcs.Action.INTERACT_WITH_AGENT.value,
            "InteractWithAgent")
        self.assertEqual(mcs.Action.INTERACT_WITH_AGENT.key, "t")
        self.assertEqual(
            mcs.Action.INTERACT_WITH_AGENT.desc,
            "Interact with an agent. If that agent has an object, "
            "it will hold out the object for you to pickup; "
            "otherwise, the agent will look sad."
            "(objectId=string, "
            "objectImageCoordsX=float, "
            "objectImageCoordsY=float)"
        )
        self.assertEqual(
            mcs.Action("InteractWithAgent"),
            mcs.Action.INTERACT_WITH_AGENT)
        self.assertEqual(mcs.Action("t"), mcs.Action.INTERACT_WITH_AGENT)

    def test_put_object(self):
        self.assertEqual(mcs.Action.PUT_OBJECT.value, "PutObject")
        self.assertEqual(mcs.Action.PUT_OBJECT.key, "7")
        self.assertEqual(
            mcs.Action.PUT_OBJECT.desc,
            "Place an object you are holding into/onto a nearby "
            "receptacle object. (objectId=string, "
            "receptacleObjectId=string, "
            "receptacleObjectImageCoordsX=float, "
            "receptacleObjectImageCoordsY=float)"
        )
        self.assertEqual(mcs.Action("PutObject"), mcs.Action.PUT_OBJECT)
        self.assertEqual(mcs.Action("7"), mcs.Action.PUT_OBJECT)

    def test_rotate_left(self):
        self.assertEqual(mcs.Action.ROTATE_LEFT.value, "RotateLeft")
        self.assertEqual(mcs.Action.ROTATE_LEFT.key, "j")
        self.assertEqual(
            mcs.Action.ROTATE_LEFT.desc,
            "Rotate your view left by 10 degrees."
        )
        self.assertEqual(mcs.Action("RotateLeft"), mcs.Action.ROTATE_LEFT)
        self.assertEqual(mcs.Action("j"), mcs.Action.ROTATE_LEFT)

    def test_rotate_right(self):
        self.assertEqual(mcs.Action.ROTATE_RIGHT.value, "RotateRight")
        self.assertEqual(mcs.Action.ROTATE_RIGHT.key, "l")
        self.assertEqual(
            mcs.Action.ROTATE_RIGHT.desc,
            "Rotate your view right by 10 degrees."
        )
        self.assertEqual(mcs.Action("RotateRight"), mcs.Action.ROTATE_RIGHT)
        self.assertEqual(mcs.Action("l"), mcs.Action.ROTATE_RIGHT)

    def test_look_up(self):
        self.assertEqual(mcs.Action.LOOK_UP.value, "LookUp")
        self.assertEqual(mcs.Action.LOOK_UP.key, "i")
        self.assertEqual(
            mcs.Action.LOOK_UP.desc,
            "Rotate your view up (subtract 10 degrees from head tilt)."
        )
        self.assertEqual(mcs.Action("LookUp"), mcs.Action.LOOK_UP)
        self.assertEqual(mcs.Action("i"), mcs.Action.LOOK_UP)

    def test_look_down(self):
        self.assertEqual(mcs.Action.LOOK_DOWN.value, "LookDown")
        self.assertEqual(mcs.Action.LOOK_DOWN.key, "k")
        self.assertEqual(
            mcs.Action.LOOK_DOWN.desc,
            "Rotate your view down (add 10 degrees to head tilt)."
        )
        self.assertEqual(mcs.Action("LookDown"), mcs.Action.LOOK_DOWN)
        self.assertEqual(mcs.Action("k"), mcs.Action.LOOK_DOWN)

    def test_pass(self):
        self.assertEqual(mcs.Action.PASS.value, "Pass")
        self.assertEqual(mcs.Action.PASS.key, " ")
        self.assertEqual(
            mcs.Action.PASS.desc,
            "Do nothing. (no params)"
        )
        self.assertEqual(mcs.Action("Pass"), mcs.Action.PASS)
        self.assertEqual(mcs.Action(" "), mcs.Action.PASS)

    def test_input_to_action_and_params(self):
        self.assertEqual(mcs.Action.input_to_action_and_params(
            'MoveBack'), ('MoveBack', {}))
        self.assertEqual(mcs.Action.input_to_action_and_params(
            'RotateRight'), ('RotateRight', {}))
        self.assertEqual(
            mcs.Action.input_to_action_and_params(
                'PickupObject,objectId=testId'
            ),
            ('PickupObject', {'objectId': 'testId'})
        )
        self.assertEqual(
            mcs.Action.input_to_action_and_params(
                'PushObject,objectId=testId,force=12.34'
            ),
            ('PushObject', {'objectId': 'testId', 'force': 12.34})
        )
        self.assertEqual(
            mcs.Action.input_to_action_and_params('Foobar'), (None, {}))
        self.assertEqual(mcs.Action.input_to_action_and_params(
            'MoveBack,key:value'), ('MoveBack', None))


if __name__ == '__main__':
    unittest.main()



import logging
from enum import Enum, unique
from typing import List, Optional

import typeguard

from .action import Action
from .stringifier import Stringifier

logger = logging.getLogger(__name__)


class GoalMetadata:
    """
    Defines metadata for a goal in the MCS 3D environment.

    Attributes
    ----------
    action_list : list of lists of (string, dict) tuples, or None
        The list of all actions that are available for the scene at each step
        (outer list). Each inner list is the list of all actions that are
        available for the single step corresponding to the inner list's index
        within the outer list. Each action is returned as a tuple containing
        the action string and the action's restricted paramters, if any.

        For example: ("Pass", {}) forces a Pass action; ("PickupObject", {})
        forces a PickupObject action with any parameters; and
        ("PickupObject", {"objectId": "a"}) forces a PickupObject action with
        the specific parameters objectId=a.

        An action_list of None means that all actions are always available.
        An empty inner list means that all actions will be available on that
        specific step.

        See :mod:`StepMetadata.action_list <machine_common_sense.StepMetadata>`
        for the available actions of the current step.
        May be a subset of all possible actions. See
        :mod:`Action <machine_common_sense.Action>`.
    category : string
        The category that describes this goal and the properties in its
        metadata. See :mod:`Goal <machine_common_sense.GoalCategory>`.
    description : string
        A human-readable sentence describing this goal and containing
        the target task(s) and object(s).

        Sizes:
        - tiny: near the size of a baseball
        - small: near the size of a baby
        - medium: near the size of a child
        - large: near the size of an adult
        - huge: near the size of a sofa

        Weights:
        - light: can be held by a baby
        - heavy: cannot be held by a baby, but can be pushed or pulled
        - massive: cannot be moved by a baby

        Colors:
        black, blue, brown, green, grey, orange, purple, red, white, yellow

        Materials:
        See :mod:`Material <machine_common_sense.Material>`.
    habituation_total : int
        The total count of habituation trials that will be in this scene.
    last_preview_phase_step : integer
        The last step of the Preview Phase of this scene, if a Preview Phase is
        scripted in the scene configuration. Each step of a Preview Phase
        normally has a single specific action defined in this goal's
        action_list property for the performer to choose, like ['Pass'].
        Default: 0 (no Preview Phase)
    last_step : integer
        The last step of this scene. This scene will automatically end
        following this step.
    metadata : dict
        The metadata specific to this goal. See
        :mod:`Goal <machine_common_sense.GoalCategory>`.
    steps_allowed_in_lava : integer
        The number of steps allowed in lava before the scene ends
    triggered_by_target_sequence : List of strings
        The target sequence the performer must open containers to trigger
        the targets placement by a placer for imitation tasks.
    """

    # Don't allow a user to call the EndHabituation action unless it's
    # specifically configured in the action_list of the scene file.
    DEFAULT_ACTIONS = [
        (item.value, {}) for item in Action
        if item not in [Action.END_HABITUATION, Action.INITIALIZE]
    ]
    # Each passive scene should have its own action_list, but if it doesn't for
    # some reason, then only allow calling Pass actions by default.
    DEFAULT_PASSIVE_SCENE_ACTIONS = [('Pass', {})]

    def __init__(
        self,
        action_list=None,
        category='',
        description='',
        habituation_total=0,
        last_preview_phase_step=0,
        last_step=None,
        metadata=None,
        steps_allowed_in_lava=0,
        triggered_by_target_sequence=None
    ):
        # The action_list must be None by default
        self.action_list = action_list
        self.category = category
        self.description = description
        self.habituation_total = habituation_total
        self.last_preview_phase_step = last_preview_phase_step
        self.last_step = last_step
        self.metadata = {} if metadata is None else metadata
        self.steps_allowed_in_lava = steps_allowed_in_lava
        self.triggered_by_target_sequence = triggered_by_target_sequence

    def __str__(self):
        return Stringifier.class_to_str(self)

    # Allows converting the class to a dictionary, along with allowing
    #   certain fields to be left out of output file
    def __iter__(self):
        yield 'action_list', self.action_list
        yield 'category', self.category
        yield 'description', self.description
        # yield 'domain_list', self.domain_list
        # yield 'info_list', self.info_list
        yield 'last_preview_phase_step', self.last_preview_phase_step
        yield 'last_step', self.last_step
        # yield 'type_list', self.type_list
        yield 'metadata', self.metadata
        yield 'triggered_by_target_sequence', self.triggered_by_target_sequence

    @typeguard.typechecked
    def retrieve_action_list_at_step(
        self,
        step_number: int,
        steps_in_lava: Optional[int] = 0,
        triggered_by_sequence_incorrect: Optional[bool] = False,
        is_passive_scene: bool = False
    ) -> List:
        """Return the action list from the given goal at the given step as a
        a list of actions tuples by default."""
        action_list = self._retrieve_unfiltered_action_list(
            step_number, steps_in_lava,
            triggered_by_sequence_incorrect, is_passive_scene)
        # remove EndHabituation parameters
        return [
            (action, params)
            if action != 'EndHabituation' else ('EndHabituation', {})
            for (action, params) in action_list
        ]

    def _retrieve_unfiltered_action_list(
        self,
        step_number: int,
        steps_in_lava: Optional[int] = 0,
        triggered_by_sequence_incorrect: Optional[bool] = False,
        is_passive_scene: bool = False
    ) -> List:
        # If steps in lava is greater than allowed, over ride
        #   action list and only return EndScene
        if steps_in_lava is not None and (
                steps_in_lava > self.steps_allowed_in_lava):
            return [("EndScene", {})]

        if triggered_by_sequence_incorrect:
            return [("EndScene", {})]

        '''Unfiltered action list from goal'''
        if self.action_list is not None:
            if step_number < self.last_preview_phase_step:
                return [('Pass', {})]
            if self.last_step is not None and step_number >= self.last_step:
                return []

            adjusted_step = step_number - self.last_preview_phase_step
            if (
                len(self.action_list) > adjusted_step and
                len(self.action_list[adjusted_step]) > 0
            ):
                return [
                    action if isinstance(action, tuple) else
                    Action.input_to_action_and_params(action)
                    for action in self.action_list[adjusted_step]
                ]

        if is_passive_scene:
            logger.warning(
                'Passive scene should have an action_list but does not; '
                'allowing only Pass actions by default.'
            )
            return GoalMetadata.DEFAULT_PASSIVE_SCENE_ACTIONS
        return GoalMetadata.DEFAULT_ACTIONS


@unique
class GoalCategory(Enum):
    """
    Each goal dict will have a "category" string that describes the type of
    scene (or, the type of task within the scene) being run. Each goal dict
    will also have a "metadata" dict containing one or more properties
    depending on the "category".
    """

    AGENTS = "agents"
    """
    In a trial that has an Agents goal, you must sit and observe a scene as one
    or more simulation-controlled agents act in predefined ways within your
    camera's viewport, and then decide whether the scene is "expected" or
    "unexpected". The camera will always be positioned at an isometric
    perspective, like you're standing on an elevated platform looking down at
    the scene. Each scene will consist of eight sequential habituation trials,
    depicting expected agent behaviors and separated by EndHabituation actions
    (each of which generates a black frame image when called), immediately
    followed by the test trial, depicting either an expected or unexpected
    agent behavior. All nine of these trials happen within the same "scene".
    These trials will demand a "common sense" understanding of agents, their
    behaviors, and their interactions with objects in the environment.

    This goal category is only used for the **passive/VoE agent tasks**. All
    interactive agent tasks will use either the `retrieval` or
    `multi retrieval` goal category.

    Notes
    -----
    You are required to call `controller.end_scene()` at the end of each scene
    with a continuous plausibility `rating`, from 0.0 (completely implausible)
    to 1.0 (completely plausible). You are not required to also pass it a
    `score`.
    """

    IMITATION = "imitation"
    """
    In a trial that has an imitation goal, you must imitate the actions of
    another agent in the scene to find and pickup a target object. Executing
    the same actions, on the same objects, in the same order, is of critical
    importance; if you do not imitate the actions correctly, you will be
    forced to end the scene (by calling end_scene, or using the END_SCENE
    action), without achieving the reward. In MCS Evaluation 4 and onward, the
    target object will always be a soccer ball (football), and, in MCS
    Evaluation 6, the imitated actions will always be opening containers of
    various colors and shapes (using the normal OpenObject action).

    Notes
    -----
    At `oracle` metadata level, the `metadata` dict property of this
    GoalMetadata object will contain a `target` property, which is a dict
    containing the following parameters:

    Parameters
    ----------
    id : string
        The unique objectId of the target object to retrieve.
    """

    INTUITIVE_PHYSICS = "intuitive physics"
    """
    In a trial that has an Intuitive Physics goal, you must sit and observe a
    scene as objects move across your camera's viewport, and then decide
    whether the scene is "plausible" or "implausible". These trials will demand
    a "common sense" understanding of basic ("intuitive") physics, like object
    permanence or shape constancy. Inspired by Emmanuel Dupoux's "IntPhys: A
    Benchmark for Visual Intuitive Physics Reasoning" (http://intphys.com).

    Notes
    -----
    You are required to call `controller.end_scene()` at the end of each scene
    with a binary plausibility `rating` -- either 0 (implausible) or 1
    (plausible) -- and a continuous plausibility `score` -- from 0.0
    (completely implausible) to 1.0 (completely plausible). This is also
    where you would submit any retrospective reporting on a per step basis via
    `report`.
    """

    PASSIVE = "passive"
    """
    In a trial that has a Passive goal, you must sit and observe a scene as
    action unfolds in your camera's viewport, and then decide whether the scene
    is "plausible" or "implausible". These trials will demand a "common sense"
    understanding of places, objects, or agency. This goal category covers all
    passive scenes that do not fall under the "agents" or "intuitive physics"
    categories.

    Notes
    -----
    You are required to call `controller.end_scene()` at the end of each scene
    with a binary plausibility `rating` -- either 0 (implausible) or 1
    (plausible) -- and a continuous plausibility `score` -- from 0.0
    (completely implausible) to 1.0 (completely plausible). This is also
    where you would submit any retrospective reporting on a per step basis via
    `report`.
    """

    RETRIEVAL = "retrieval"
    """
    In a trial that has a retrieval goal, you must find and pickup a target
    object. In MCS Evaluation 4 and onward, the target object will always be a
    soccer ball (football).

    This may involve exploring the scene, avoiding obstacles,
    interacting with objects (like closed containers) or agents, and tracking
    moving objects. These trials will demand a "common sense" understanding of
    self navigation (how to move and rotate yourself within a scene and around
    obstacles), object interaction (how objects work, including opening
    containers), the basic physics of movement (kinematics, gravity, friction,
    etc.), and agency (identifying people and using them to achieve a goal).

    Notes
    -----
    At `oracle` metadata level, the `metadata` dict property of this
    GoalMetadata object will contain a `target` property, which is a dict
    containing the following parameters:

    Parameters
    ----------
    id : string
        The unique objectId of the target object to retrieve.
    """

    MULTI_RETRIEVAL = "multi retrieval"
    """
    In a trial that has a multi retrieval goal, you must find and pickup one or
    more target objects. In MCS Evaluation 4 and onward, the target object will
    always be a soccer ball (football).

    This may involve exploring the scene, avoiding obstacles,
    interacting with objects (like closed containers) or agents, and tracking
    moving objects. These trials will demand a "common sense" understanding of
    self navigation (how to move and rotate yourself within a scene and around
    obstacles), object interaction (how objects work, including opening
    containers), the basic physics of movement (kinematics, gravity, friction,
    etc.), and agency (identifying people and using them to achieve a goal).

    Notes
    -----
    At `oracle` metadata level, the `metadata` dict property of this
    GoalMetadata object will contain a `targets` property, which is a list of
    dicts that each contain the following parameters:

    Parameters
    ----------
    id : string
        The unique objectId of one of the target objects to retrieve.
    """



from typing import Dict, List, Optional

import typeguard
from shapely import geometry

from .goal_metadata import GoalCategory, GoalMetadata

GOAL_ACHIEVED = 1
GOAL_NOT_ACHIEVED = 0
STEP_PENALTY = 0.001
LAVA_PENALTY = 100.0


class Reward(object):
    '''Reward utility class'''
    @staticmethod
    def __get_object_from_list(objects: List[Dict],
                               target_id: str) -> Dict:
        '''
        Finds an object in a list. Uses a generator to return the first item
        or defaults to None if the target isn't found.

        Args:
            objects: list of object dictionaries
            target_id: str objectId of the object to find

        Returns:
            target: object dictionary if found or None
        '''
        return next((o for o in objects if o['objectId'] == target_id), None)

    @staticmethod
    def _convert_object_to_planar_polygon(
            goal_object: Dict) -> geometry.Polygon:
        '''
        Project goal object bounds (x,y,z) to an XZ planar polygon.

        Args:
            goal_object: dict

        Returns:
            polygons: shapely.geometry.Polygon
        '''
        bbox3d = goal_object['objectBounds']['objectBoundsCorners']
        # project to XZ plane
        xz_pts = [(pt['x'], pt['z']) for pt in bbox3d]
        return geometry.MultiPoint(xz_pts).convex_hull

    @staticmethod
    def _calc_retrieval_reward(
            goal: GoalMetadata,
            objects: Dict,
            agent: Dict,
            performer_reach: float,
            goal_reward: float = GOAL_ACHIEVED) -> float:
        '''
        Calculate the reward for the retrieval goal.

        The goal object must be in the agent's hand.

        Args:
            goal: GoalMetadata
            objects: Dict
            agent: Dict
            performer_reach: float
            goal_reward: float

        Returns:
            float: 1 for goal achieved and no goal_reward arg is given,
            goal_reward value if goal_reward arg is given, 0 otherwise.

        '''
        reward = GOAL_NOT_ACHIEVED
        goal_objects = []

        metadata = goal.metadata or {}
        # Different goal categories may use different property names
        target_names = ['target', 'targets']
        for target_name in target_names:
            # Some properties may be dicts, and some may be lists of dicts
            targets = metadata.get(target_name) or []
            targets = targets if isinstance(targets, list) else [targets]
            for target in targets:
                goal_id = target.get('id')
                goal_object = Reward.__get_object_from_list(objects, goal_id)
                if goal_object:
                    goal_objects.append(goal_object)

        # Only attain the reward if all targets are picked up
        if goal_objects and all(
                [obj.get('isPickedUp', False) for obj in goal_objects]):
            reward = goal_reward

        return reward

    @staticmethod
    def _adjust_score_penalty(
            current_score: int,
            number_steps: int,
            steps_on_lava: int,
            lava_penalty: float = LAVA_PENALTY,
            step_penalty: float = STEP_PENALTY) -> float:
        '''
        Calculate the score penalty based on the number of steps,
        if the current step results in a reward being achieved do
        not penalize them for the step that resulted in the goal
        being achieved.

        Args:
            current_score: 1 or 0 depending if reward achieved
            number_steps: the current step count
            steps_on_lava: the number of total steps on lava
            lava_penalty: the point deduction for each step on lava
            step_penalty: point deduction for each step

        Returns:
            float: new score based off of step penalty

        '''
        step_penalty = STEP_PENALTY if step_penalty is None else step_penalty
        lava_penalty = LAVA_PENALTY if lava_penalty is None else lava_penalty
        if current_score == 1:
            return current_score - ((number_steps - 1) * float(step_penalty))

        if steps_on_lava is None:
            steps_on_lava = 0
        return current_score - (number_steps * float(step_penalty)) - \
            (steps_on_lava * float(lava_penalty))

    @staticmethod
    def _calculate_default_reward(
            goal: GoalMetadata,
            objects: Dict,
            agent: Dict,
            reach: float,
            goal_reward: float = GOAL_ACHIEVED) -> float:
        '''Returns the default reward of 0; not achieved.'''
        return GOAL_NOT_ACHIEVED

    @staticmethod
    @typeguard.typechecked
    def calculate_reward(
            goal: Optional[GoalMetadata],
            objects: List[Dict],
            agent: Dict,
            number_steps: int,
            reach: Optional[float],
            steps_on_lava: Optional[int] = None,
            lava_penalty: Optional[float] = LAVA_PENALTY,
            step_penalty: Optional[float] = STEP_PENALTY,
            goal_reward: Optional[float] = GOAL_ACHIEVED) -> float:
        '''
        Determine if the agent achieved the goal.

        Args:
            goal: GoalMetadata
            objects: Dict
            agent: Dict
            reach: float
            steps_on_lava: int
            lava_penalty: float
            step_penalty: float
            goal_reward: float

        Returns:
            int: reward is 1 if goal achieved, 0 otherwise

        '''

        category = None
        if goal is not None and goal.metadata:
            category = goal.metadata.get('category', None)

        switch = {
            GoalCategory.IMITATION.value: Reward._calc_retrieval_reward,
            GoalCategory.RETRIEVAL.value: Reward._calc_retrieval_reward,
            GoalCategory.MULTI_RETRIEVAL.value: Reward._calc_retrieval_reward
        }

        current_score = switch.get(category, Reward._calculate_default_reward)(
            goal,
            objects,
            agent,
            reach,
            GOAL_ACHIEVED if goal_reward is None else goal_reward
        )

        return Reward._adjust_score_penalty(
            current_score, number_steps, steps_on_lava, lava_penalty,
            step_penalty
        )



from runner_script import MultipleFileRunnerScript


def action_callback(scene_data, step_metadata, runner_script):
    if step_metadata.step_number < 36:
        return 'RotateRight', {}
    return None, None


def main():
    MultipleFileRunnerScript('Interactive Scenes - Rotate', action_callback)


if __name__ == "__main__":
    main()



import json
import pathlib
import random
import unittest
from typing import Tuple

import PIL

from machine_common_sense.recorder import (BaseRecorder, ImageRecorder,
                                           JsonRecorder, VideoRecorder)


class ConcreteBaseRecorder(BaseRecorder):
    '''Test implementation of the BaseRecorder'''

    def _write(self):
        raise NotImplementedError


class TestBaseRecorder(unittest.TestCase):

    def setUp(self):
        self.recorder = ConcreteBaseRecorder()
        self.recorder.start()

    def tearDown(self):
        self.recorder.finish()

    def test_recording(self):
        '''Recorder automatically starts (i.e. becomes recording)'''
        self.assertTrue(self.recorder.recording)

    def test_thread(self):
        '''Thread should start automatically'''
        self.assertTrue(self.recorder._thread.is_alive())

    def test_queue_empty(self):
        '''No frames sitting in the queue'''
        self.assertTrue(self.recorder._queue.empty())

    def test_num_recorded(self):
        '''No frames have been written yet'''
        self.assertEqual(self.recorder.num_recorded, 0)

    def test_flush(self):
        '''Flushing an empty recorder queue should not be a problem'''
        self.recorder.flush()
        self.assertTrue(self.recorder._queue.empty())

    def test_empty_finish(self):
        '''wrap up the recorder without writing anything'''
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertTrue(self.recorder._queue.empty())
        self.assertEqual(self.recorder.num_recorded, 0)


class TestJsonRecorder(unittest.TestCase):
    json_prefix = "test_json_"
    test_json_file = pathlib.Path("tests/test_json_{:04d}.json")

    def setUp(self):
        self.recorder = JsonRecorder(self.test_json_file)

    def tearDown(self):
        '''Delete all unit test output images'''
        for p in pathlib.Path('tests').glob('test_json_*'):
            p.unlink()

    def test_data(self, val=4):
        return {"test1": "t1", "obj": {"o1": "o2"},
                "arr": [1, 2, 3], "val": val}

    def test_write(self):
        output_file = pathlib.Path("tests/test_json_0000.json")
        data = self.test_data()
        self.recorder.add(data)
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertEqual(self.recorder.num_recorded, 1)
        self.assertTrue(self.recorder._queue.empty())
        self.assertTrue(output_file.is_file())
        self.assertTrue(output_file.parent.is_dir())
        with open(output_file, 'r') as json_file:
            actual = json.load(json_file)
        self.assertEqual(actual, data)

    def test_multiple_write(self):
        output_file1 = pathlib.Path("tests/test_json_0000.json")
        output_file2 = pathlib.Path("tests/test_json_0001.json")
        output_file3 = pathlib.Path("tests/test_json_0002.json")
        data1 = self.test_data(1)
        data2 = self.test_data(2)
        data3 = self.test_data(3)
        self.recorder.add(data1)
        self.recorder.add(data2)
        self.recorder.add(data3)
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertEqual(self.recorder.num_recorded, 3)
        self.assertTrue(self.recorder._queue.empty())
        self.assertTrue(output_file1.is_file())
        self.assertTrue(output_file1.parent.is_dir())
        with open(output_file1, 'r') as json_file:
            actual = json.load(json_file)
        self.assertEqual(actual, data1)
        with open(output_file2, 'r') as json_file:
            actual = json.load(json_file)
        self.assertEqual(actual, data2)
        with open(output_file3, 'r') as json_file:
            actual = json.load(json_file)
        self.assertEqual(actual, data3)

    def test_missing_folder(self):
        '''ImageRecorder requires existing path'''
        bad_recorder = JsonRecorder(
            pathlib.Path("missing/test_json_{:04d}.json"))
        bad_recorder.add(self.test_data())
        with self.assertRaises(FileNotFoundError):
            bad_recorder.finish()

    def test_add_after_finish(self):
        output_file = pathlib.Path("tests/test_json_0000.json")
        self.recorder.add(self.test_data())
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertEqual(self.recorder.num_recorded, 1)
        self.assertTrue(self.recorder._queue.empty())
        self.assertTrue(output_file.is_file())
        self.assertTrue(output_file.parent.is_dir())
        self.recorder.add(self.test_data())
        self.assertTrue(self.recorder._queue.empty())


class TestImageRecorder(unittest.TestCase):
    image_prefix = "test_image_"
    test_image_file = pathlib.Path("tests/test_image_{:04d}.jpg")

    def setUp(self):
        self.recorder = ImageRecorder(self.test_image_file)

    def tearDown(self):
        '''Delete all unit test output images'''
        for p in pathlib.Path('tests').glob('test_image_*'):
            p.unlink()

    def random_color(self) -> Tuple:
        '''Create a random 8-bit RGB tuple'''
        r = random.randint(0, 255)
        g = random.randint(0, 255)
        b = random.randint(0, 255)
        return (r, g, b)

    def test_write(self):
        output_file = pathlib.Path("tests/test_image_0000.jpg")
        size = (50, 100)
        img = PIL.Image.new("RGB", size, self.random_color())
        self.recorder.add(img)
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertEqual(self.recorder.num_recorded, 1)
        self.assertTrue(self.recorder._queue.empty())
        self.assertTrue(output_file.is_file())
        self.assertTrue(output_file.parent.is_dir())

    def test_write_png(self):
        '''ImageRecorder can save image sequences in png format'''
        png_format = pathlib.Path("tests/test_image_{:04d}.png")
        png_recorder = ImageRecorder(png_format)
        output_file = pathlib.Path("tests/test_image_0000.png")
        size = (50, 100)
        img = PIL.Image.new("RGB", size, self.random_color())
        png_recorder.add(img)
        png_recorder.finish()
        self.assertFalse(png_recorder.recording)
        self.assertEqual(png_recorder.num_recorded, 1)
        self.assertTrue(png_recorder._queue.empty())
        self.assertTrue(output_file.is_file())
        self.assertTrue(output_file.parent.is_dir())

    def test_multiple_write(self):
        output_file = pathlib.Path("tests/test_image_0000.jpg")
        size = (50, 100)
        img = PIL.Image.new("RGB", size, self.random_color())
        self.recorder.add(img)
        img = PIL.Image.new("RGB", size, self.random_color())
        self.recorder.add(img)
        img = PIL.Image.new("RGB", size, self.random_color())
        self.recorder.add(img)
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertEqual(self.recorder.num_recorded, 3)
        self.assertTrue(self.recorder._queue.empty())
        self.assertTrue(output_file.is_file())
        self.assertTrue(output_file.parent.is_dir())

    def test_multiple_write_different_sizes(self):
        '''ImageRecorder can write different size images'''
        first_file = pathlib.Path("tests/test_image_0000.jpg")
        second_file = pathlib.Path("tests/test_image_0001.jpg")
        size = (50, 100)
        other_size = (100, 100)
        img = PIL.Image.new("RGB", size, self.random_color())
        self.recorder.add(img)
        img = PIL.Image.new("RGB", other_size, self.random_color())
        self.recorder.add(img)
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertEqual(self.recorder.num_recorded, 2)
        self.assertTrue(self.recorder._queue.empty())
        self.assertTrue(first_file.is_file())
        self.assertTrue(second_file.is_file())

    def test_missing_folder(self):
        '''ImageRecorder requires existing path'''
        bad_recorder = ImageRecorder(
            pathlib.Path("missing/test_image_{:04d}.jpg"))
        size = (50, 100)
        img = PIL.Image.new("RGB", size, self.random_color())
        bad_recorder.add(img)
        with self.assertRaises(FileNotFoundError):
            bad_recorder.finish()

    def test_add_after_finish(self):
        output_file = pathlib.Path("tests/test_image_0000.jpg")
        size = (50, 100)
        img = PIL.Image.new("RGB", size, self.random_color())
        self.recorder.add(img)
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertEqual(self.recorder.num_recorded, 1)
        self.assertTrue(self.recorder._queue.empty())
        self.assertTrue(output_file.is_file())
        self.assertTrue(output_file.parent.is_dir())
        self.recorder.add(img)
        self.assertTrue(self.recorder._queue.empty())


class TestVideoRecorder(unittest.TestCase):

    test_video_file = pathlib.Path('tests/test.mp4')
    fps = 30

    def setUp(self):
        self.recorder = VideoRecorder(
            vid_path=self.test_video_file,
            fps=self.fps
        )

    def tearDown(self):
        if self.test_video_file.exists():
            self.test_video_file.unlink()

    def test_video_path(self):
        self.assertEqual(self.recorder.path, self.test_video_file)

    def test_fps(self):
        self.assertEqual(self.recorder.fps, self.fps)

    def test_default_fourcc(self):
        self.assertEqual(self.recorder.fourcc, 'mp4v')

    def test_video_writer_none(self):
        '''The actual writer is None until the first frame'''
        self.assertIsNone(self.recorder.writer)

    def test_add(self):
        size = (50, 100)
        img = PIL.Image.new("RGB", size)
        self.recorder.add(img)
        self.assertFalse(self.recorder._queue.empty())
        self.assertIsNotNone(self.recorder.writer)

    def test_add_flush(self):
        size = (50, 100)
        img = PIL.Image.new("RGB", size)
        self.recorder.add(img)
        self.recorder.add(img)
        self.recorder.add(img)
        self.assertFalse(self.recorder._queue.empty())
        self.recorder.flush()
        self.assertEqual(self.recorder.num_recorded, 3)
        self.assertTrue(self.recorder.recording)
        self.assertTrue(self.recorder._queue.empty())

    def test_finish(self):
        nframes = 100
        size = (50, 100)
        img = PIL.Image.new("RGB", size)
        for _ in range(nframes):
            self.recorder.add(img)
        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertTrue(self.recorder._queue.empty())
        self.assertEqual(self.recorder.num_recorded, nframes)

    def test_wrong_size_frame(self):
        # the first frame established the video recorder size
        size = (50, 100)
        img = PIL.Image.new("RGB", size)
        self.recorder.add(img)

        # any different sized frame will cause a ValueError exception
        wrong_size = (100, 100)
        wrong_size_img = PIL.Image.new("RGB", wrong_size, color='red')
        with self.assertRaises(ValueError):
            self.recorder.add(wrong_size_img)

        self.recorder.finish()
        self.assertFalse(self.recorder.recording)
        self.assertTrue(self.recorder._queue.empty())
        self.assertEqual(self.recorder.num_recorded, 1)


if __name__ == '__main__':
    unittest.main()



from runner_script import MultipleFileRunnerScript

import machine_common_sense as mcs

experience_with_room_action_list = [
    # Spin around, and then rotate to face the first container/corner.
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    # Move ahead to the first container/corner.
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    # Look down into the container, and then back up.
    'LookDown', 'LookDown', 'LookDown', 'LookDown', 'LookDown',
    'LookUp', 'LookUp', 'LookUp', 'LookUp', 'LookUp',
    # Rotate to face the second container/corner.
    'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight',
    'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight',
    'RotateRight', 'RotateRight',
    # Move ahead to the second container/corner (along a short wall).
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    # Rotate to face the second container/corner.
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft',
    # Look down into the container, and then back up.
    'LookDown', 'LookDown', 'LookDown', 'LookDown', 'LookDown',
    'LookUp', 'LookUp', 'LookUp', 'LookUp', 'LookUp',
    # Rotate to face the third container/corner.
    'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight',
    'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight',
    'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight',
    # Move ahead to the third container/corner (along a long wall).
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead',
    # Rotate to face the third container/corner.
    'RotateLeft', 'RotateLeft', 'RotateLeft',
    # Look down into the container, and then back up.
    'LookDown', 'LookDown', 'LookDown', 'LookDown', 'LookDown',
    'LookUp', 'LookUp', 'LookUp', 'LookUp', 'LookUp',
    # Rotate to face the fourth container/corner.
    'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight',
    'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight', 'RotateRight',
    'RotateRight', 'RotateRight',
    # Move ahead to the fourth container/corner (along a long wall).
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    # Rotate to face the fourth container/corner.
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft',
    # Look down into the container, and then back up.
    'LookDown', 'LookDown', 'LookDown', 'LookDown', 'LookDown',
    'LookUp', 'LookUp', 'LookUp', 'LookUp', 'LookUp',
    # Pass until the first kidnap event, and then pass during the drop event.
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass',
    'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass', 'Pass'
    'Pass',
    # Spin around after the final kidnap event.
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft', 'RotateLeft',
    'RotateLeft'
]

move_to_target_action_list = [
    # Move ahead to the target container/corner.
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead', 'MoveAhead',
    # Look down to the container.
    'LookDown', 'LookDown', 'LookDown'
]

scene_state = {}


def action_callback(scene_data, step_metadata, runner_script):
    global scene_state

    if step_metadata.step_number == 0:
        scene_state = {
            'action_list': [],
            'move_back_count': 0,
            'open': False,
            'pickup': False,
            'target_id': None
        }
        kidnap_rotation = int(
            scene_data['goal']['sceneInfo']['kidnapRotation']
        )
        rotate_right_count = int((360 - kidnap_rotation) / 10)
        target_corner = scene_data['debug']['chosenCorner']
        if target_corner == 'back_left':
            rotate_right_count -= 15
        elif target_corner == 'back_right':
            rotate_right_count += 15
        elif target_corner == 'front_left':
            rotate_right_count -= 3
        elif target_corner == 'front_right':
            rotate_right_count += 3
        else:
            raise ValueError(f'The chosenCorner is not valid: {target_corner}')
        rotate_right_count %= 36
        if rotate_right_count > 18:
            rotate_right_count -= 36
        scene_state['target_id'] = (
            scene_data['goal']['metadata']['target']['id']
        )
        scene_state['action_list'].extend(
            experience_with_room_action_list.copy() + ([
                f"Rotate{'Left' if rotate_right_count < 0 else 'Right'}"
            ] * rotate_right_count) + move_to_target_action_list.copy()
        )

    if len(step_metadata.action_list) == 1:
        return step_metadata.action_list[0]

    if len(scene_state['action_list']) > step_metadata.step_number:
        return mcs.Action.input_to_action_and_params(
            scene_state['action_list'][step_metadata.step_number]
        )

    if not scene_state['open']:
        scene_state['open'] = True
        return 'OpenObject', {
            'objectImageCoordsX': 300,
            'objectImageCoordsY': 200
        }

    if scene_state['open'] and step_metadata.return_status == 'OBSTRUCTED':
        scene_state['open'] = False
        scene_state['move_back_count'] += 1
        return 'MoveBack', {}

    if not scene_state['pickup']:
        scene_state['pickup'] = True
        return 'PickupObject', {'objectId': scene_state['target_id']}

    if scene_state['pickup'] and step_metadata.return_status != 'SUCCESSFUL':
        scene_state['pickup'] = False
        if scene_state['move_back_count'] > 0:
            scene_state['move_back_count'] -= 1
            return 'MoveAhead', {}

    return None, None


def main():
    MultipleFileRunnerScript('Reorientation Scenes', action_callback)


if __name__ == "__main__":
    main()



import unittest

import machine_common_sense as mcs
from machine_common_sense.action import FORCE_ACTIONS, OBJECT_MOVE_ACTIONS
from machine_common_sense.config_manager import ConfigManager
from machine_common_sense.controller import DEFAULT_MOVE
from machine_common_sense.parameter import (Parameter, compare_param_values,
                                            rebuild_endhabituation)


class TestParameter(unittest.TestCase):

    sc = {'screenshot': False,
          'version': 2,
          'floorTextures': [],
          'isometric': False,
          'performerStart': {'rotation': {'y': 0.0,
                                          'z': 0.0,
                                          'x': 0.0},
                             'position': {'y': 0.0,
                                          'z': 0.0,
                                          'x': 0.0}},
          'observation': False,
          'intuitivePhysics': False,
          'floorMaterial': 'AI2-THOR/Materials/Fabrics/CarpetWhite 3',
          'ceilingMaterial': 'AI2-THOR/Materials/Walls/Drywall',
          'holes': [],
          'lava': [],
          'objects': [{'shows': [{'position': {'y': 0.25,
                                               'z': 1.0,
                                               'x': 0.25},
                                  'boundingBox': [],
                                  'stepBegin': 0,
                                  'scale': {'y': 0.5,
                                            'z': 0.5,
                                            'x': 0.5},
                                  'rotation': {'y': 0.0,
                                               'z': 0.0,
                                               'x': 0.0}}],
                       'pickupable': False,
                       'mass': 1.0,
                       'materials': ['AI2-THOR/Materials/Plastics/BlueRubber'],
                       'salientMaterials': ['rubber'],
                       'moveable': False,
                       'type': 'ball',
                       'id': 'testBall1'},
                      {'shows': [{'position': {'y': 0.25,
                                               'z': 2.0,
                                               'x': -0.25},
                                  'boundingBox': [],
                                  'stepBegin': 0,
                                  'scale': {'y': 0.5,
                                            'z': 0.5,
                                            'x': 0.5},
                                  'rotation': {'y': 0.0,
                                               'z': 0.0,
                                               'x': 0.0}}],
                       'pickupable': False,
                       'mass': 1.0,
                       'materials': ['AI2-THOR/Materials/Plastics/BlueRubber'],
                       'salientMaterials': ['rubber'],
                       'moveable': False,
                       'type': 'ball',
                       'id': 'testBall2'}],
          'wallMaterial': 'AI2-THOR/Materials/Walls/Drywall',
          'name': 'depth and segmentation test',
          'roomDimensions': {'y': 3.0,
                             'z': 10.0,
                             'x': 10.0}}

    def setUp(self):
        config = ConfigManager(config_file_or_dict={})
        self.parameter_converter = Parameter(config)

    def test_initialization_wrap_step(self):
        wrapped_step = self.parameter_converter.wrap_step(
            output_folder="path",
            action='Initialize',
            sceneConfig=self.sc)
        # sceneConfig does not get removed
        self.assertIsNotNone(wrapped_step.get('sceneConfig'))

    def test_initialization_build_ai2thor_step(self):
        wrapped_step, params = self.parameter_converter.build_ai2thor_step(
            output_path="path",
            action='Initialize',
            sceneConfig=self.sc)
        # sceneConfig gets removed
        self.assertIsNone(wrapped_step.get('sceneConfig'))

    def test_wrap_step_action(self):
        actual = self.parameter_converter.wrap_step(
            output_folder="path",
            action="TestAction",
            numberProperty=1234,
            stringProperty="test_property")
        expected = {
            "action": "TestAction",
            "continuous": True,
            "gridSize": 0.1,
            "logs": True,
            "numberProperty": 1234,
            "renderDepthImage": False,
            "renderObjectImage": False,
            "snapToGrid": False,
            "stringProperty": "test_property",
            "consistentColors": False,
            "recordTopDown": False,
            "topDownImagePath": "path"
        }
        self.assertEqual(actual, expected)

    def test_wrap_step_metadata_oracle(self):
        config = ConfigManager(config_file_or_dict={'metadata': 'oracle'})
        parameter_converter = Parameter(config)
        actual = parameter_converter.wrap_step(
            output_folder="path",
            action="TestAction",
            numberProperty=1234,
            stringProperty="test_property")
        # Changed depth and object because oracle should result in both being
        # true.
        expected = {
            "action": "TestAction",
            "continuous": True,
            "gridSize": 0.1,
            "logs": True,
            "numberProperty": 1234,
            "renderDepthImage": True,
            "renderObjectImage": True,
            "snapToGrid": False,
            "stringProperty": "test_property",
            "consistentColors": True,
            "recordTopDown": False,
            "topDownImagePath": "path"
        }
        self.assertEqual(actual, expected)

    def test_wrap_step_metadata_level2(self):
        config = ConfigManager(config_file_or_dict={'metadata': 'level2'})
        parameter_converter = Parameter(config)
        actual = parameter_converter.wrap_step(
            output_folder="path",
            action="TestAction",
            numberProperty=1234,
            stringProperty="test_property")
        # Changed depth and object because oracle should result in both being
        # true.
        expected = {
            "action": "TestAction",
            "continuous": True,
            "gridSize": 0.1,
            "logs": True,
            "numberProperty": 1234,
            "renderDepthImage": True,
            "renderObjectImage": True,
            "snapToGrid": False,
            "stringProperty": "test_property",
            "consistentColors": False,
            "recordTopDown": False,
            "topDownImagePath": "path"
        }
        self.assertEqual(actual, expected)

    def test_wrap_step_metadata_level1(self):
        config = ConfigManager(config_file_or_dict={'metadata': 'level1'})
        parameter_converter = Parameter(config)
        actual = parameter_converter.wrap_step(
            output_folder="path",
            action="TestAction",
            numberProperty=1234,
            stringProperty="test_property")
        # Changed depth and object because oracle should result in both being
        # true.
        expected = {
            "action": "TestAction",
            "continuous": True,
            "gridSize": 0.1,
            "logs": True,
            "numberProperty": 1234,
            "renderDepthImage": True,
            "renderObjectImage": False,
            "snapToGrid": False,
            "stringProperty": "test_property",
            "consistentColors": False,
            "recordTopDown": False,
            "topDownImagePath": "path"
        }
        self.assertEqual(actual, expected)

    def test_generate_noise(self):
        min_noise = self.parameter_converter.MIN_NOISE
        max_noise = self.parameter_converter.MAX_NOISE

        current_noise = self.parameter_converter._generate_noise()
        self.assertTrue(min_noise <= current_noise <= max_noise)

    def test_get_amount(self):
        # default amounts
        amount = self.parameter_converter._get_amount(
            action=OBJECT_MOVE_ACTIONS[0])
        self.assertIsInstance(amount, float)
        self.assertAlmostEqual(amount, Parameter.DEFAULT_OBJECT_MOVE_AMOUNT)

        amount = self.parameter_converter._get_amount(
            action=FORCE_ACTIONS[0])
        self.assertIsInstance(amount, float)
        self.assertAlmostEqual(amount, Parameter.DEFAULT_AMOUNT)

        # amount of None equates to default for any action
        amount = self.parameter_converter._get_amount(
            action=OBJECT_MOVE_ACTIONS[0], amount=None)
        self.assertIsInstance(amount, float)
        self.assertAlmostEqual(amount, Parameter.DEFAULT_AMOUNT)

        amount = self.parameter_converter._get_amount(
            action=FORCE_ACTIONS[0], amount=None)
        self.assertIsInstance(amount, float)
        self.assertAlmostEqual(amount, Parameter.DEFAULT_AMOUNT)

        # ensure exceptions are raised for unexpected value or types
        with self.assertRaises(ValueError):
            self.parameter_converter._get_amount(
                action=mcs.Action.PASS, amount="string")

        with self.assertRaises(ValueError):
            self.parameter_converter._get_amount(
                action=mcs.Action.PASS, amount=1.1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_amount(
                action=mcs.Action.PASS, amount=-0.1)

    def test_get_force(self):
        force = self.parameter_converter._get_force(
            action=mcs.Action.PULL_OBJECT, force=1)
        self.assertIsInstance(force, float)
        self.assertAlmostEqual(force, 1.0)

        force = self.parameter_converter._get_force(
            action=mcs.Action.PULL_OBJECT)
        self.assertIsInstance(force, float)
        self.assertAlmostEqual(force, Parameter.DEFAULT_AMOUNT)

        # defaults for none force regardless of action type
        force = self.parameter_converter._get_force(
            action=mcs.Action.PUSH_OBJECT, force=None)
        self.assertIsInstance(force, float)
        self.assertAlmostEqual(force, Parameter.DEFAULT_AMOUNT)

        force = self.parameter_converter._get_force(
            action=mcs.Action.TORQUE_OBJECT, force=None)
        self.assertIsInstance(force, float)
        self.assertAlmostEqual(force, Parameter.DEFAULT_AMOUNT)

        # raise errors when force value and type is unexpected
        with self.assertRaises(ValueError):
            self.parameter_converter._get_force(
                action=mcs.Action.PASS, force="string")

        with self.assertRaises(ValueError):
            self.parameter_converter._get_force(
                action=mcs.Action.PASS, force=1.1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_force(
                action=mcs.Action.PASS, force=-0.1)

        # torque force ranges
        with self.assertRaises(ValueError):
            self.parameter_converter._get_force(
                action=mcs.Action.TORQUE_OBJECT, force=1.1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_force(
                action=mcs.Action.TORQUE_OBJECT, force=-1.1)

    def test_get_clockwise(self):
        clockwise = self.parameter_converter._get_clockwise(clockwise="False")
        self.assertIsInstance(clockwise, bool)
        self.assertEqual(clockwise, False)

        clockwise = self.parameter_converter._get_clockwise(clockwise="false")
        self.assertIsInstance(clockwise, bool)
        self.assertEqual(clockwise, False)

        clockwise = self.parameter_converter._get_clockwise(clockwise="True")
        self.assertIsInstance(clockwise, bool)
        self.assertEqual(clockwise, True)

        clockwise = self.parameter_converter._get_clockwise(clockwise="true")
        self.assertIsInstance(clockwise, bool)
        self.assertEqual(clockwise, True)

        clockwise = self.parameter_converter._get_clockwise(clockwise=False)
        self.assertIsInstance(clockwise, bool)
        self.assertEqual(clockwise, False)

        clockwise = self.parameter_converter._get_clockwise(clockwise=True)
        self.assertIsInstance(clockwise, bool)
        self.assertEqual(clockwise, True)

        clockwise = self.parameter_converter._get_clockwise()
        self.assertIsInstance(clockwise, bool)
        self.assertEqual(clockwise, True)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_clockwise(clockwise="string")

        with self.assertRaises(ValueError):
            self.parameter_converter._get_clockwise(clockwise=1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_clockwise(clockwise=-0.1)

    def test_get_movement_direction(self):
        (lateral, straight) = (
            self.parameter_converter._get_movement_direction())
        self.assertIsInstance(lateral, int)
        self.assertIsInstance(straight, int)
        self.assertEqual(lateral, 0)
        self.assertEqual(straight, 1)

        (lateral, straight) = (
            self.parameter_converter._get_movement_direction(
                lateral=-1))
        self.assertIsInstance(lateral, int)
        self.assertIsInstance(straight, int)
        self.assertEqual(lateral, -1)
        self.assertEqual(straight, 0)

        (lateral, straight) = (
            self.parameter_converter._get_movement_direction(
                straight=-1))
        self.assertIsInstance(lateral, int)
        self.assertIsInstance(straight, int)
        self.assertEqual(lateral, 0)
        self.assertEqual(straight, -1)

        (lateral, straight) = (
            self.parameter_converter._get_movement_direction(
                lateral=1, straight=-1))
        self.assertIsInstance(lateral, int)
        self.assertIsInstance(straight, int)
        self.assertEqual(lateral, 1)
        self.assertEqual(straight, -1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral=2)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                straight=-2)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral=-2, straight=1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral=-1, straight=2)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral=0.1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                straight=0.1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral=-0.1, straight=1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral=1, straight=-0.1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral="x", straight="z")

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral="x", straight=1)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral=1, straight="z")

        with self.assertRaises(ValueError):
            self.parameter_converter._get_movement_direction(
                lateral=True, straight=False)

    def test_get_number(self):
        number = self.parameter_converter._get_number(key="val", val=7)
        self.assertEqual(number, 7)
        self.assertIsInstance(number, float)

        number = self.parameter_converter._get_number(key="val", not_val=7)
        self.assertIsNone(number)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_number(
                key="string", string="invalid"
            )

    def test_get_number_with_default(self):
        number = self.parameter_converter._get_number_with_default(
            key="val", default=5, val=7)
        self.assertEqual(number, 7)
        self.assertIsInstance(number, float)

        number = self.parameter_converter._get_number_with_default(
            key="val", default=5, not_val=7)
        self.assertEqual(number, 5)
        self.assertIsInstance(number, float)

        with self.assertRaises(ValueError):
            self.parameter_converter._get_number_with_default(
                key="string", default=5, string="invalid")

        number = self.parameter_converter._get_number_with_default(
            key="val", default=5, val=None
        )
        self.assertEqual(number, 5)
        self.assertIsInstance(number, float)

    def test_get_move_magnitude(self):
        magnitude = self.parameter_converter._get_move_magnitude(
            action="",
            force=0.5,
            amount=0.7
        )
        self.assertIsInstance(magnitude, float)
        self.assertEqual(magnitude, DEFAULT_MOVE)

        magnitude = self.parameter_converter._get_move_magnitude(
            action=FORCE_ACTIONS[0],
            force=0.5,
            amount=0.7
        )
        self.assertIsInstance(magnitude, float)
        self.assertEqual(magnitude, 0.5)

        magnitude = self.parameter_converter._get_move_magnitude(
            action=OBJECT_MOVE_ACTIONS[0],
            force=0.5,
            amount=0.7
        )
        self.assertIsInstance(magnitude, float)
        self.assertEqual(magnitude, 0.7)

    def test_get_teleport(self):
        (teleport_rot, teleport_pos) = self.parameter_converter._get_teleport()
        self.assertIsNone(teleport_rot)
        self.assertIsNone(teleport_pos)

        (teleport_rot, teleport_pos) = self.parameter_converter._get_teleport(
            yRotation=90
        )
        self.assertEqual(teleport_rot, {'y': 90.0})
        self.assertIsNone(teleport_pos)

        (teleport_rot, teleport_pos) = self.parameter_converter._get_teleport(
            xPosition=1, zPosition=2)
        self.assertIsNone(teleport_rot)
        self.assertEqual(teleport_pos, {'x': 1.0, 'z': 2.0})

        (teleport_rot, teleport_pos) = self.parameter_converter._get_teleport(
            yRotation=90, xPosition=1, zPosition='2')
        self.assertEqual(teleport_rot, {'y': 90.0})
        self.assertEqual(teleport_pos, {'x': 1.0, 'z': 2.0})

        with self.assertRaises(ValueError):
            self.parameter_converter._get_teleport(
                yRotation='invalid',
                xPosition='1',
                zPosition='2')

        with self.assertRaises(ValueError):
            self.parameter_converter._get_teleport(
                yRotation='90',
                xPosition='invalid',
                zPosition='2'
            )

        with self.assertRaises(ValueError):
            self.parameter_converter._get_teleport(
                yRotation=90,
                xPosition=1,
                zPosition='invalid'
            )

    def test_teleport_position(self):
        position = self.parameter_converter._get_teleport_position(
            xPosition=1, zPosition=2)
        self.assertIsInstance(position, dict)
        self.assertIn('x', position)
        self.assertIn('z', position)
        self.assertIsInstance(position['x'], float)
        self.assertIsInstance(position['z'], float)
        self.assertEqual(position['x'], 1)
        self.assertEqual(position['z'], 2)

        position = self.parameter_converter._get_teleport_position(
            xPosition=None, zPosition=None)
        self.assertIsNone(position)

        position = self.parameter_converter._get_teleport_position(
            xPosition=None, zPosition=2)
        self.assertIsNone(position)

        position = self.parameter_converter._get_teleport_position(
            xPosition=1, zPosition=None)
        self.assertIsNone(position)

    def test_teleport_rotation(self):
        rotation = self.parameter_converter._get_teleport_rotation(
            yRotation=45)
        self.assertIsInstance(rotation, dict)
        self.assertIn('y', rotation)
        self.assertIsInstance(rotation['y'], float)
        self.assertEqual(rotation['y'], 45)

        rotation = self.parameter_converter._get_teleport_rotation(
            yRotation=None)
        self.assertIsNone(rotation)

    def test_convert_y_image_coord_for_unity(self):
        image_coord = 0
        unity_coord = \
            self.parameter_converter._convert_y_image_coord_for_unity(
                image_coord)
        screen_height = self.parameter_converter.config.get_screen_height()
        self.assertEqual(unity_coord, screen_height - 1)

        image_coord = screen_height - 1  # screen height is out of bounds
        unity_coord = \
            self.parameter_converter._convert_y_image_coord_for_unity(
                image_coord)
        self.assertEqual(unity_coord, 0)

        image_coord = screen_height
        with self.assertRaises(ValueError):
            self.parameter_converter._convert_y_image_coord_for_unity(
                y_coord=image_coord
            )

        # value much greater than the screen height
        image_coord = 1000000
        with self.assertRaises(ValueError):
            self.parameter_converter._convert_y_image_coord_for_unity(
                y_coord=image_coord
            )

        # value less than the origin
        image_coord = -1
        with self.assertRaises(ValueError):
            self.parameter_converter._convert_y_image_coord_for_unity(
                y_coord=image_coord
            )

        # Invalid image coordinate
        image_coord = None
        with self.assertRaises(TypeError):
            self.parameter_converter._convert_y_image_coord_for_unity(
                y_coord=image_coord
            )

        # Invalid image coordinate value
        image_coord = "one"
        with self.assertRaises(TypeError):
            self.parameter_converter._convert_y_image_coord_for_unity(
                y_coord=image_coord
            )

    def test_mcs_action_to_ai2thor_action(self):
        ai2thor_action = \
            self.parameter_converter._mcs_action_to_ai2thor_action(
                mcs.Action.OPEN_OBJECT)
        self.assertIsInstance(ai2thor_action, str)
        self.assertEqual(ai2thor_action, "MCSOpenObject")

        ai2thor_action = \
            self.parameter_converter._mcs_action_to_ai2thor_action(
                mcs.Action.CLOSE_OBJECT)
        self.assertIsInstance(ai2thor_action, str)
        self.assertEqual(ai2thor_action, "MCSCloseObject")

        ai2thor_action = \
            self.parameter_converter._mcs_action_to_ai2thor_action(
                mcs.Action.DROP_OBJECT)
        self.assertIsInstance(ai2thor_action, str)
        self.assertEqual(ai2thor_action, "DropHandObject")

    def test_compare_param_values(self):
        self.assertTrue(compare_param_values('', ''))
        self.assertTrue(compare_param_values('a', 'a'))

        self.assertTrue(compare_param_values('1', '1'))
        self.assertTrue(compare_param_values('1.0', '1.0'))
        self.assertTrue(compare_param_values('1.234', '1.234'))
        self.assertTrue(compare_param_values(1, 1))
        self.assertTrue(compare_param_values(1.0, 1.0))
        self.assertTrue(compare_param_values(1.234, 1.234))

        self.assertTrue(compare_param_values('1', 1))
        self.assertTrue(compare_param_values(1, '1'))
        self.assertTrue(compare_param_values('1.0', 1.0))
        self.assertTrue(compare_param_values(1.0, '1.0'))

        self.assertTrue(compare_param_values('1', 1.0))
        self.assertTrue(compare_param_values(1.0, '1'))
        self.assertTrue(compare_param_values(1, '1.0'))
        self.assertTrue(compare_param_values('1.0', 1))

        self.assertTrue(compare_param_values('1', '1.0'))
        self.assertTrue(compare_param_values('1.0', '1'))
        self.assertTrue(compare_param_values(1, 1.0))
        self.assertTrue(compare_param_values(1.0, 1))

        self.assertTrue(compare_param_values('1.234', 1.234))
        self.assertTrue(compare_param_values(1.234, '1.234'))
        self.assertTrue(compare_param_values('1.234', 1.234))
        self.assertTrue(compare_param_values(1.234, '1.234'))

        self.assertFalse(compare_param_values('a', '1'))
        self.assertFalse(compare_param_values('1', 'a'))

        self.assertFalse(compare_param_values('a', '1.0'))
        self.assertFalse(compare_param_values('1.0', 'a'))
        self.assertFalse(compare_param_values('a', '1.234'))
        self.assertFalse(compare_param_values('1.234', 'a'))

        self.assertFalse(compare_param_values('1', '1.234'))
        self.assertFalse(compare_param_values('1.234', '1'))
        self.assertFalse(compare_param_values('1.0', '1.234'))
        self.assertFalse(compare_param_values('1.234', '1.0'))

        self.assertFalse(compare_param_values(1, 1.234))
        self.assertFalse(compare_param_values(1.234, 1))
        self.assertFalse(compare_param_values(1.0, 1.234))
        self.assertFalse(compare_param_values(1.234, 1.0))

    def test_rebuild_endhabituation(self):
        # EndHabituation rebuilds parameters from the goal list
        endhabit_goal = (mcs.Action.END_HABITUATION.value, {})
        self.assertEqual(
            "EndHabituation",
            rebuild_endhabituation(
                step_action_list=[endhabit_goal]
            )
        )

        endhabit_goal = (mcs.Action.END_HABITUATION.value, {"yRotation": 90})
        self.assertEqual(
            "EndHabituation,yRotation=90",
            rebuild_endhabituation(
                step_action_list=[endhabit_goal]
            )
        )

        endhabit_goal = (
            mcs.Action.END_HABITUATION.value, {
                'xPosition': 0.0, 'zPosition': 0.0})
        self.assertEqual(
            "EndHabituation,xPosition=0.0,zPosition=0.0",
            rebuild_endhabituation(
                step_action_list=[endhabit_goal]
            )
        )

        endhabit_goal = (
            mcs.Action.END_HABITUATION.value,
            {'xPosition': 0.0, 'zPosition': 0.0, 'yRotation': 0.0})
        self.assertEqual(
            "EndHabituation,xPosition=0.0,zPosition=0.0,yRotation=0.0",
            rebuild_endhabituation(
                step_action_list=[endhabit_goal]
            )
        )



import logging
from pathlib import Path

from .controller_events import AbstractControllerSubscriber
from .recorder import JsonRecorder
from .stringifier import Stringifier

logger = logging.getLogger(__name__)


class ControllerLogger(AbstractControllerSubscriber):
    '''Handles debugging and user output based on controller events
    '''

    def on_start_scene(self, payload):
        logger.debug(
            f"STARTING NEW SCENE: {payload.scene_config.name}")
        logger.debug(
            f"METADATA TIER: {payload.config.get_metadata_tier().value}")
        logger.debug(f"STEP: {payload.step_number}")
        logger.debug("ACTION: Initialize")

        self._write_debug_output(payload)

    def on_before_step(self, payload):
        logger.debug("================================================"
                     "===============================")
        logger.debug(f"STEP: {str(payload.step_number)}")
        logger.debug(f'ACTION: {payload.action}')
        if payload.goal.habituation_total >= payload.habituation_trial:
            logger.debug(
                f'HABITUATION TRIAL: {payload.habituation_trial} / '
                f'{payload.goal.habituation_total}'
            )

        elif payload.goal.habituation_total > 0:
            logger.debug("HABITUATION TRIAL: DONE")
        else:
            logger.debug("HABITUATION TRIAL: NONE")

    def on_after_step(self, payload):
        self._write_debug_output(payload)

    def _write_debug_output(self, payload):
        step_output = payload.restricted_step_output
        logger.debug(f"RETURN STATUS: {step_output.return_status}")
        logger.debug(f"REWARD: {str(step_output.reward)}")
        logger.debug("SELF METADATA:")
        logger.debug(f"  CAMERA HEIGHT: {str(step_output.camera_height)}")
        logger.debug(f"  HEAD TILT: {str(step_output.head_tilt)}")
        logger.debug(f"  POSITION: {str(step_output.position)}")
        logger.debug(f"  ROTATION: {str(step_output.rotation)}")
        logger.debug(f"  HAPTIC FEEDBACK: {str(step_output.haptic_feedback)}")
        logger.debug(f"OBJECTS: {len(step_output.object_list)} TOTAL")
        if len(step_output.object_list) > 0:
            for line in Stringifier.generate_pretty_object_output(
                    step_output.object_list):
                logger.debug(f"    {line}")


class ControllerDebugFileGenerator(AbstractControllerSubscriber):
    '''Handles writing mcs output debug files
    '''

    def on_start_scene(self, payload):
        self._write_debug_output_file(payload)

    def on_after_step(self, payload):
        self._write_debug_output_file(payload)

    def _write_debug_output_file(self, payload):
        step_output = \
            payload.restricted_step_output.copy_without_depth_or_images()
        if payload.output_folder and payload.config.is_save_debug_json():
            with open(
                ((f'{payload.output_folder}mcs_output_' +
                  str(payload.step_number)) + '.json'), 'w') as json_file:
                json_file.write(str(step_output))


class ControllerAi2thorFileGenerator(AbstractControllerSubscriber):
    '''Handles writing AI2Thor debug files
    '''

    def on_start_scene(self, payload):
        if payload.output_folder and payload.config.is_save_debug_json():
            path = Path(payload.output_folder) / 'ai2thor_input_{}.json'
            self._in_recorder = JsonRecorder(json_template=path)
            path = Path(payload.output_folder) / 'ai2thor_output_{}.json'
            self._out_recorder = JsonRecorder(json_template=path)
        self._write_debug_input_file(payload)
        self._write_debug_output_file(payload)

    def on_after_step(self, payload):
        self._write_debug_input_file(payload)
        self._write_debug_output_file(payload)

    def _write_debug_input_file(self, payload):
        data = payload.wrapped_step
        if self._in_recorder:
            self._in_recorder.add(data)

    def _write_debug_output_file(self, payload):
        data = {
            "metadata": payload.step_metadata.metadata
        }
        if self._out_recorder:
            self._out_recorder.add(data)



import shutil
import unittest
from pathlib import Path
from zipfile import ZIP_DEFLATED, ZipFile

from machine_common_sense.unity_executable_provider import (
    AbstractExecutionCache, Downloader, UnityExecutableProvider)

TEST_TMP = "./tmp"
TEST_CACHE_LOCATION = "./tmp/.mcs-test"
TEST_ZIP = "./tmp/test.zip"


class MockExecutionCache(AbstractExecutionCache):
    EXECUTABLE_FILE = "MCS-AI2-THOR-Unity-App-v.x86_64"
    GZ_FILES = []
    REQUIRED_FILES = ["MCS-AI2-THOR-Unity-App-v.x86_64"]

    def _get_executable_file(self):
        return self.EXECUTABLE_FILE

    def _get_gz_files(self):
        return self.GZ_FILES

    def _get_required_files(self):
        return self.REQUIRED_FILES


class MockDownloader(Downloader):
    count = 0

    def download(self, url: str, filename: str,
                 destination_folder: Path) -> Path:
        self.count = self.count + 1
        path = shutil.copy(
            TEST_ZIP, (destination_folder / filename).as_posix())
        return Path(path)


class TestUnityExecutableProvider(unittest.TestCase):

    def setUp(self):
        AbstractExecutionCache.CACHE_LOCATION = Path(TEST_CACHE_LOCATION)
        self.provider = UnityExecutableProvider()
        self.cache = MockExecutionCache()
        self.provider._cache = self.cache
        self.provider._downloader = MockDownloader()
        file = Path(TEST_ZIP)
        zip = ZipFile(file, 'w', ZIP_DEFLATED)
        executable = Path(TEST_TMP) / self.cache.EXECUTABLE_FILE
        executable.touch()
        zip.write(executable, self.cache.EXECUTABLE_FILE)
        zip.close()

    def tearDown(self):
        shutil.rmtree(TEST_CACHE_LOCATION)
        shutil.rmtree(TEST_TMP)

    def test_provider_init(self):
        self.assertTrue(Path(TEST_CACHE_LOCATION).exists())

    def test_get_executable_empty(self):
        result = self.provider.get_executable("test")
        self.assertTrue(result.exists())

    def test_get_executable_cache(self):
        self.assertEqual(self.provider._downloader.count, 0)
        result = self.provider.get_executable("test2")
        self.assertTrue(result.exists())
        self.assertEqual(self.provider._downloader.count, 1)

        # try again and verify the downloader isn't called again.
        result = self.provider.get_executable("test2")
        self.assertTrue(result.exists())
        self.assertEqual(self.provider._downloader.count, 1)

    def test_culling(self):
        self.assertEqual(self.provider._downloader.count, 0)
        result1 = self.provider.get_executable("test1")
        self.assertTrue(result1.exists())
        self.assertEqual(self.provider._downloader.count, 1)

        # try again and verify the downloader isn't called again.
        result2 = self.provider.get_executable("test2")
        self.assertTrue(result2.exists())
        self.assertEqual(self.provider._downloader.count, 2)
        self.assertFalse(result1.exists())
        self.assertFalse(result1.parent.exists())


if __name__ == '__main__':
    unittest.main()



import datetime
import glob
import logging
import platform
import shutil
import tarfile
from abc import ABC, abstractmethod
from pathlib import Path
from zipfile import ZipFile

import requests
from progressbar import Bar, FileTransferSpeed, Percentage, ProgressBar

from ._version import __version__

logger = logging.getLogger(__name__)

LINUX_URL = "https://github.com/NextCenturyCorporation/MCS/releases/download/{ver}/MCS-AI2-THOR-Unity-App-v{ver}-linux.zip"  # noqa
MAC_URL = "https://github.com/NextCenturyCorporation/MCS/releases/download/{ver}/MCS-AI2-THOR-Unity-App-v{ver}-mac.zip"  # noqa
LINUX_DEV_URL = "https://ai2thor-unity-releases.s3.amazonaws.com/MCS-AI2-THOR-Unity-App-vdevelop-linux.zip"  # noqa
MAC_DEV_URL = "https://ai2thor-unity-releases.s3.amazonaws.com/MCS-AI2-THOR-Unity-App-vdevelop-mac.zip"  # noqa


class UnityExecutableProvider():
    '''Automatically provides MCS AI2-THOR Unity executable for the MCS package.
    Will check a cache and download if necessary'''

    DOWNLOAD_FILE = "MCS-AI2-THOR-Unity-App-v{}.zip"
    PLATFORM_MAC = "Darwin"
    PLATFORM_LINUX = "Linux"
    PLATFORM_OTHER = "other"

    def __init__(self):
        self._downloader = Downloader()
        self._platform_init()

    def _platform_init(self):
        self._switcher = {
            self.PLATFORM_LINUX: self._linux_init,
            self.PLATFORM_MAC: self._mac_init,
            self.PLATFORM_OTHER: self._other_init
        }
        sys = platform.system()
        self._switcher.get(sys, self.PLATFORM_OTHER)()

    def _mac_init(self):
        self._cache = MacExecutionCache()

    def _linux_init(self):
        self._cache = LinuxExecutionCache()

    def _other_init(self):
        raise Exception(
            "Ai2thorProvider only supports Linux and Mac. "
            f"Platform={platform.system()}"
        )

    def clear_cache(self, version=None):
        '''clears the entire cache if no version is passed in, otherwise
        clears the specified version'''
        if version is None:
            self._cache.remove_whole_cache()
        else:
            self._cache.remove_cache_version(version)

    def get_executable(self, version=None,
                       download_if_missing=True, force_download=False) -> Path:
        '''For a given version, this will return the path to that executable
        or throw an exception if it cannot be found.
        '''
        if version is None:
            version = __version__
        if version in ["dev", "development", "develop"]:
            logger.warn(
                "Warning: Attempting to use development version of "
                "MCS-AI2Thor.  This is intended for developers only.")
            version = "develop"
            url = self._downloader.get_url(version)
            force_download |= self._downloader.is_updated(
                url, self._cache.modified_date(version))
        if not force_download and self._cache.has_version(version):
            return self._cache.get_execution_location(version)
        if (download_if_missing or force_download):
            url = self._downloader.get_url(version)
            dest = self._cache._get_version_dir(version)
            dest.mkdir(exist_ok=True, parents=True)
            zip_file_name = self.DOWNLOAD_FILE.format(version)
            zip_file = self._downloader.download(url, zip_file_name, dest)
            self._cache.add_zip_to_cache(version, zip_file)
            return self._cache.get_execution_location(version)
        else:
            raise Exception(
                f"unable to locate ai2thor for version={version}")


class AbstractExecutionCache(ABC):
    '''Handles platform agnostic (between Mac and Linux) code for running a
    cache for MCS Unity executables.  '''
    CACHE_LOCATION = Path.home() / ".mcs/"
    TIMESTAMP_FILE = ".timestamp"

    def __init__(self):
        cache_base = self.CACHE_LOCATION.expanduser()
        self._base = cache_base
        if not cache_base.exists():
            cache_base.mkdir(parents=True)
            logger.debug(
                f"Created mcs cache at {cache_base.as_posix()}")

    def get_execution_location(self, version: str) -> Path:
        '''Returns the location of the executable file'''
        ver_dir = self._get_version_dir(version)
        exec = ver_dir / self._get_executable_file().format(
            version=version)
        self.cull_cache(version)
        return exec

    def _get_version_dir(self, version: str) -> Path:
        '''returns a Path object for the path to the directory for the given
         version.
        '''
        return self._base / version

    def has_version(self, version: str) -> bool:
        '''Return whether the cache has the given version'''
        ver_dir = self._get_version_dir(version)
        if not ver_dir.exists():
            return False
        for file in self._get_required_files():
            file = file.format(version=version)
            exists = (ver_dir / file).exists()
            if not exists:
                logger.info(f"Missing file: {file}")
                return False
        return True

    def modified_date(self, version: str) -> datetime.datetime:
        mod_date = datetime.datetime.min
        if (self.has_version(version)):
            ver_dir = self._get_version_dir(version)
            path = (ver_dir / self.TIMESTAMP_FILE)
            if path.exists():
                return datetime.datetime.fromtimestamp(
                    path.stat().st_mtime)
        return mod_date

    def add_zip_to_cache(self, version, zip_file: Path):
        ver_dir = self._get_version_dir(version)
        zip = ZipFile(zip_file)
        logger.info(
            f"Unzipping {zip_file.name} to {ver_dir.as_posix()}")
        zip.extractall(ver_dir)
        logger.info(f"Deleting {zip_file.name}.")
        zip_file.unlink()
        ver_dir = self._get_version_dir(version)
        file = self._get_executable_file().format(version=version)
        (ver_dir / file).chmod(755)

        for file in self._get_gz_files():
            file = file.format(version=version)
            gz = ver_dir / file
            logger.info(
                f"Unzipping {gz.name} to {ver_dir.as_posix()}.")
            self._unzip_tar_gz(gz, ver_dir)
            logger.info(f"Deleting {gz.name}.")
            gz.unlink()
        (ver_dir / self.TIMESTAMP_FILE).touch()

    def _unzip_tar_gz(self, tar_gz_file: Path, dest: Path):
        tar = tarfile.open(tar_gz_file.as_posix(), "r:gz")
        tar.extractall(path=dest.as_posix())
        tar.close()

    def remove_cache_version(self, version):
        # todo need to delete files
        ver_dir = self._get_version_dir(version)
        shutil.rmtree(ver_dir)

    def remove_whole_cache(self):
        shutil.rmtree(self._base)

    def cull_cache(self, last_version: str):
        try:
            list = glob.glob(f"{self._base.as_posix()}/*")
            ver_dir = self._get_version_dir(last_version)
            for name in list:
                logger.debug(f"test: {name}")
                file = Path(name)
                keep = file.is_dir() and ver_dir.samefile(file)
                if not keep:
                    logger.debug(f"deleting {file.as_posix()}")
                    shutil.rmtree(file)
        except Exception:
            logger.exception(
                "Error attempting to cull MCS Unity executable cache", )

    @abstractmethod
    def _get_executable_file(self):
        pass

    @abstractmethod
    def _get_gz_files(self):
        pass

    @abstractmethod
    def _get_required_files(self):
        pass


class MacExecutionCache(AbstractExecutionCache):
    '''Handles Mac specific code for running a cache for MCS Unity executables.
    '''
    EXECUTABLE_FILE = "MCS-AI2-THOR-Unity-App-v{version}.app/Contents/MacOS/MCS-AI2-THOR"  # noqa
    REQUIRED_FILES = ["MCS-AI2-THOR-Unity-App-v{version}.app"]
    GZ_FILES = []

    def _get_executable_file(self):
        return self.EXECUTABLE_FILE

    def _get_gz_files(self):
        return self.GZ_FILES

    def _get_required_files(self):
        return self.REQUIRED_FILES


class LinuxExecutionCache(AbstractExecutionCache):
    '''Handles Linux specific code for running a cache for MCS Unity executables.
    '''
    REQUIRED_FILES = [
        "LinuxPlayer_s.debug",
        "UnityPlayer.so",
        "UnityPlayer_s.debug",
        "MCS-AI2-THOR-Unity-App-v{version}_Data",
        "MCS-AI2-THOR-Unity-App-v{version}.x86_64"]
    EXECUTABLE_FILE = "MCS-AI2-THOR-Unity-App-v{version}.x86_64"
    GZ_FILES = ["MCS-AI2-THOR-Unity-App-v{version}_Data.tar.gz"]

    def _get_executable_file(self):
        return self.EXECUTABLE_FILE

    def _get_gz_files(self):
        return self.GZ_FILES

    def _get_required_files(self):
        return self.REQUIRED_FILES


class Downloader():
    '''Handles downloading MCS AI2THOR package'''

    def get_url(self, ver):
        sys = platform.system()
        if (sys == "Windows"):
            raise Exception("Windows is not supported")
        elif sys == "Linux":
            return LINUX_URL.format(
                ver=ver) if ver != "develop" else LINUX_DEV_URL
        elif sys == "Darwin":
            return MAC_URL.format(ver=ver) if ver != "develop" else MAC_DEV_URL
        else:
            raise Exception(f"OS '{sys}' not supported")

    def download(self, url: str, filename: str,
                 destination_folder: Path) -> Path:
        file_data = self._do_download(url)
        # todo probably change this
        file = destination_folder / filename
        file.write_bytes(file_data)
        return file

    # TODO should probably change how we download so we can write file
    # sequentially
    def _do_download(self, url):
        logger.debug(f"Downloading file from {url}")
        r = requests.get(url, stream=True)
        r.raise_for_status()
        size = int(r.headers["Content-Length"].strip())
        total_bytes = 0

        widgets = [
            url.split("/")[-1],
            ": ",
            Bar(),
            Percentage(),
            " ",
            FileTransferSpeed(),
            f" of {str(round(size / 1024 / 1024, 2))[:4]}MB",
        ]

        pbar = ProgressBar(widgets=widgets, maxval=size).start()
        file_data = []
        for buf in r.iter_content(1024):
            if buf:
                file_data.append(buf)
                total_bytes += len(buf)
                pbar.update(total_bytes)
        pbar.finish()

        return b"".join(file_data)

    def is_updated(self, url, date: datetime.datetime):
        # Default to true?
        try:
            updated = True
            r = requests.get(url, stream=True)
            r.raise_for_status()
            last_mod = r.headers['last-modified']
            r.close()
            dt = datetime.datetime.strptime(
                last_mod,
                "%a, %d %b %Y %H:%M:%S %Z")
            updated = dt > date
        except Exception as e:
            logger.warn(
                "Error checking last modified of development build",
                exc_info=e)
        finally:
            return updated



import unittest

import numpy as np
from numpy.testing import assert_array_equal

from machine_common_sense.controller_media import convert_depth_to_hsv


class TestControllerMedia(unittest.TestCase):

    def test_convert_depth_to_hsv_hue(self):
        diff = 0.375 / 256.0
        input_array = np.array([np.arange(0, 0.375, diff)])
        actual_output = convert_depth_to_hsv(input_array, 150.0)
        expected_output = np.array([[[i, 55, 255] for i in range(0, 256)]])
        assert_array_equal(actual_output, expected_output)

    def test_convert_depth_to_hsv_sat_and_val(self):
        input_array = np.array([
            np.arange(i, i + 3.75, 0.375) for i in np.arange(0, 150, 3.75)
        ])
        actual_output = convert_depth_to_hsv(input_array, 150.0)
        expected_output = np.reshape(np.array([
            [[0, i, 255] for i in range(55, 256)] +
            [[0, 255, i] for i in reversed(range(56, 255))]
        ]), (40, 10, 3))
        assert_array_equal(actual_output, expected_output)


if __name__ == '__main__':
    unittest.main()



import argparse

import machine_common_sense as mcs
from machine_common_sense.controller import Controller
from machine_common_sense.logging_config import LoggingConfig
from machine_common_sense.scripts.run_interactive_scenes_follow_path import \
    PathFollower

commands = []


def parse_args():
    parser = argparse.ArgumentParser(description='Run MCS')
    parser.add_argument(
        'mcs_scene_json_file',
        help='MCS JSON scene configuration file to load')
    parser.add_argument(
        '--mcs_unity_build_file',
        type=str,
        default=None,
        help='Path to MCS unity build file')
    parser.add_argument(
        '--mcs_unity_version',
        type=str,
        default=None,
        help='version of MCS Unity executable.  Default: current')
    parser.add_argument(
        '--config_file_path',
        type=str,
        default=None,
        required=True,
        help='Path to configuration file to read in and set various '
        'properties, such as metadata level and whether or not to '
        'save history files properties.')
    return parser.parse_args()


path_follower = PathFollower()


def run_scene(controller: Controller, scene_data, path):

    print("Resetting the current scene...")
    output = controller.start_scene(scene_data)

    action = 'start'
    while True:
        action, params = path_follower.action_callback(
            scene_data, output, None)
        if action is None:
            break
        controller.step(action, **params)

    controller.end_scene()


def main():
    mcs.init_logging(LoggingConfig.get_dev_logging_config())
    args = parse_args()
    scene_data = mcs.load_scene_json_file(args.mcs_scene_json_file)

    path = scene_data.get('debug', {}).get('path')
    if not path:
        print("Scene did not have 'debug.path' section")
        return

    controller = mcs.create_controller(
        unity_app_file_path=args.mcs_unity_build_file,
        config_file_or_dict=args.config_file_path,
        unity_cache_version=args.mcs_unity_version)

    path_follower.init_callback(controller)

    scene_file_path = args.mcs_scene_json_file
    scene_file_name = scene_file_path[scene_file_path.rfind('/') + 1:]

    if 'name' not in scene_data.keys():
        scene_data['name'] = scene_file_name[0:scene_file_name.find('.')]

    if controller is not None:
        run_scene(controller, scene_data, path)


if __name__ == "__main__":
    main()



from .stringifier import Stringifier


class ObjectMetadata(object):
    """
    Defines metadata for an object in the MCS 3D environment.

    Attributes
    ----------
    uuid : string
        The unique ID of this object, used with some actions.
    associated_with_agent: str
        The agent holding this object.
    dimensions : list of dicts
        The dimensions of this object in the environment's 3D global
        coordinate system as a list of 8 points (dicts with "x", "y", and "z").
    direction : dict
        The direction vector of "x", "y", and "z" degrees between your position
        and this object's position (the difference in the two positions),
        normalized to 1. You can use the "x" and "y" as the "rotation" and
        "horizon" parameters (respectively) in a "RotateLook" action to face
        this object.
    distance : float
        DEPRECATED. Same as distance_in_steps. Please use distance_in_steps
        or distance_in_world.
    distance_in_steps : float
        The distance from you to this object in number of steps ("Move"
        actions) on the 2D X/Z movement grid.
    distance_in_world : float
        The distance from you to this object in the environment's 3D global
        coordinate system.
    held : boolean
        Whether you are holding this object.
    is_open : boolean
        Whether the object is open or not
    locked: boolean
        Whether the object is locked
    mass : float
        Haptic feedback. The mass of this object.
    material_list : list of strings
        Haptic feedback. The material(s) of this object.
        See :mod:`Material <machine_common_sense.Material>`.
    openable : boolean
        Whether the object can be opened
    position : dict
        The "x", "y", and "z" coordinates for the global position of the
        center of this object's 3D model.
    rotation : dict
        This object's rotation angles around the "x", "y", and "z" axes
        in degrees.
    segment_color : dict
        The "r", "g", and "b" pixel values of this object in images from the
        StepMetadata's "object_mask_list".
    shape : string
        This object's shape in plain English.
    simulation_agent_held_object: str
        The object associated held for the simulation agent that the performer
        can retrieve through the InteractWithAgent method.
    simulation_agent_is_holding_held_object: str
        Whether this simulation agent is currently holding its
        associated held object.
    state_list : list of strings
        This object's state(s) from the current step in the scene. Sometimes
        used by objects with scripted behavior in passive scenes.
    texture_color_list : list of strings
        This object's colors, derived from its textures, in plain English.
    visible : boolean
        Whether you can see this object in your camera viewport.
    """

    def __init__(
        self,
        uuid="",
        dimensions=None,
        direction=None,
        distance=-1.0,
        distance_in_steps=-1.0,
        distance_in_world=-1.0,
        held=False,
        mass=0.0,
        material_list=None,
        position=None,
        rotation=None,
        segment_color=None,
        shape="",
        state_list=None,
        texture_color_list=None,
        visible=False,
        is_open=False,
        openable=False,
        locked=False,
        associated_with_agent="",
        simulation_agent_held_object="",
        simulation_agent_is_holding_held_object=False
    ):
        self.uuid = uuid
        self.dimensions = [] if dimensions is None else dimensions
        self.direction = {} if direction is None else direction
        self.distance = distance
        self.distance_in_steps = distance_in_steps
        self.distance_in_world = distance_in_world
        self.held = held
        self.mass = mass
        self.material_list = [] if material_list is None else material_list
        self.position = {} if position is None else position
        self.rotation = {} if rotation is None else rotation
        self.segment_color = {} \
            if segment_color is None \
            else segment_color
        self.shape = shape
        self.state_list = [] if state_list is None else state_list
        self.texture_color_list = (
            [] if texture_color_list is None else texture_color_list
        )
        self.visible = visible
        self.is_open = is_open
        self.openable = openable
        self.locked = locked
        self.associated_with_agent = associated_with_agent
        self.simulation_agent_held_object = simulation_agent_held_object
        self.simulation_agent_is_holding_held_object = \
            simulation_agent_is_holding_held_object

    def __str__(self):
        return Stringifier.class_to_str(self)

    # Allows converting the class to a dictionary, along with allowing
    #   certain fields to be left out of output file
    def __iter__(self):
        yield 'uuid', self.uuid
        yield 'dimensions', self.dimensions
        yield 'direction', self.direction
        yield 'distance', self.distance
        yield 'distance_in_steps', self.distance_in_steps
        yield 'distance_in_world', self.distance_in_world
        yield 'held', self.held
        yield 'mass', self.mass
        yield 'material_list', self.material_list
        yield 'position', self.position
        yield 'rotation', self.rotation
        yield 'segment_color', self.segment_color
        yield 'shape', self.shape
        yield 'state_list', self.state_list
        yield 'texture_color_list', self.texture_color_list
        yield 'visible', self.visible
        yield 'is_open', self.is_open
        yield 'openable', self.openable
        yield 'locked', self.locked
        yield 'associated_with_agent', self.associated_with_agent
        yield 'simulation_agent_held_object', self.simulation_agent_held_object
        yield 'simulation_agent_is_holding_held_object', \
            self.simulation_agent_is_holding_held_object
